# -*- coding: utf-8 -*-
import time
import os
import datetime
import hmac
import hashlib

from cdecimal import *
from delorean import Delorean
from collections import OrderedDict, defaultdict

from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange import order_types
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse

logger = get_logger(__name__)


class QuadrigaBTCCADExchange(ExchangeAPIWrapper):
    """
    Test Order ID (should be cancelled):
        c11tajt6k7ilo4oasjgodzpb0u28y4nb5o5g8ymt65eu9352mmq0lfbl9du8hhkn
    """

    def __init__(self, session=None, configuration=None):
        super(QuadrigaBTCCADExchange, self).__init__(session)

        self.name = u'QUADRIGA_BTC_CAD'
        self.friendly_name = u'Quadriga CX BTC-CAD'
        self.base_url = 'https://api.quadrigacx.com/v2'
        self.currency = u'CAD'
        self.volume_currency = u'BTC'
        self.bid_string = '0'
        self.ask_string = '1'

        # Configurables with defaults.
        self.fee = Decimal('0.0025')
        self.market_order_fee = self.fee
        self.limit_order_fee = self.fee
        self.fiat_balance_tolerance = Money('0.0001', 'CAD')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.max_tick_speed = 1
        self.min_order_size = Money('0.0025', 'BTC')
        self.use_cached_orderbook = False

        if configuration:
            self.configure(configuration)

    def req(self, req_method, url, **kwargs):
        req = super(QuadrigaBTCCADExchange, self).req(req_method, url, **kwargs)
        time.sleep(0.2) # Unfortunately necessary due to nonce errors.

        return req

    def resp(self, req):
        response = super(QuadrigaBTCCADExchange, self).resp(req)

        try:
            if 'error' in response:
                errors_string = response['error']['message']

                if 'exceeds available' in errors_string:
                    raise exceptions.InsufficientFundsError()
                elif 'Cannot perform request - not found' in errors_string:
                    raise exceptions.CancelOrderNotFoundError()
                elif 'Nonce' in errors_string:
                    raise exceptions.NonceError()
                else:
                    logger.info(response['error'])
                    raise exceptions.ExchangeAPIErrorException(self, errors_string)

            return response
        except KeyError:
            raise exceptions.ExchangeAPIFailureException(self, response)

    def transactions_req(self):
        payload = {
            'limit': 100,
        }

        return self.req('post', '/user_transactions', data=payload)
    
    def transactions_resp(self, req):
        return self.resp(req)
        
    def transactions(self):
        return self.transactions_resp(self.transactions_req()) 
    
    def trades(self):
        raw_transactions =  self.transactions()
        trades = []

        for raw_transaction in raw_transactions:
            # It is a trade if an order_id is present in the transaction.
            if int(raw_transaction['type']) == 2:
                btc = Money(raw_transaction['btc'], 'BTC')
                fiat = Money(raw_transaction[self.currency.lower()], self.currency)

                # Bids look like this: {'cad':'-5075.50','btc':'15.77622250'}
                # and asks like this:  {'cad':'256.88','btc':'-0.80000000'}
                if btc > 0: # Bid
                    # Bids have BTC fees
                    fee = Money(raw_transaction['fee'], 'BTC')
                    # Quadriga returns amounts which have fees removed
                    # we need to add them back in to keep our accounting accurate
                    btc += fee

                    if fiat > 0:
                        raise exceptions.ExchangeAPIErrorException(
                            self,
                            'Trade #%s does not have opposite btc and fiat signs' % (
                            raw_transaction['id'],
                        ))

                    fiat = abs(fiat)
                elif btc < 0:  # Ask.
                    # Asks have CAD fees.
                    fee = Money(raw_transaction['fee'], self.currency)
                    fiat += fee

                    if fiat < 0:
                        raise exceptions.ExchangeAPIErrorException(
                            self,
                            'Trade #%s does not have opposite btc and fiat signs' % (
                            raw_transaction['id'],
                        ))

                    btc = abs(btc)
                else:
                    raise exceptions.ExchangeAPIErrorException(
                        self,
                        'Zero volume Trade #%s, so API doesn\'t tell us the side.' % ( 
                        raw_transaction['id'],
                    ))

                trades.append({
                    'time': self._datetime_to_timestamp(raw_transaction['datetime']),
                    'trade_id': unicode(raw_transaction['id']),
                    'order_id': unicode(raw_transaction['order_id']),
                    'btc': btc,
                    'fiat': fiat,
                    'fee': fee,
                })

        return trades
    
    def trades_for_order(self, order_id):
        trades = self.trades()
        return [t for t in trades if t['order_id'] == order_id]

    def _datetime_to_timestamp(self, dt_string):
        # As of 2015-12-01 13:00 UTC, quadriga timestamps are now in UTC
        # (with no warning or backwards compatability, of course)
        return int(parse(dt_string).epoch)

    ######## Common Exchange Methods ########

    # modifies request_args
    def auth_request(self, req_method, url, request_args):
        try:
            self.api_key
            self.secret
            self.client_id
        except AttributeError:
            self.api_key = self._load_env('QUADRIGA_BTC_CAD_KEY')
            self.client_id = self._load_env('QUADRIGA_BTC_CAD_CLIENT_ID')
            self.secret = self._load_env('QUADRIGA_BTC_CAD_SECRET')

        try:
            payload = request_args['data']
        except KeyError:
            payload = request_args['data'] = {}

        nonce = unicode(int(round(time.time() * 1000)))
        message= nonce + self.api_key + self.client_id

        sig = hmac.new(
            hashlib.md5(self.secret).hexdigest(),
            msg=message,
            digestmod=hashlib.sha256,
        ).hexdigest().upper()

        payload.update({
            'nonce':nonce,
            'key': self.api_key,
            'signature':sig
        })
        
    def get_balance_req(self):
        return self.req('post', '/balance')

    def get_balance_resp(self, req):
        response = self.resp(req)

        try:
            btc_balance = Money(response['btc_balance'], 'BTC')
            cad_balance = Money(response['cad_balance'], 'CAD')
        except KeyError:
            raise exceptions.ExchangeAPIFailureException(self, response)

        data = Balance()
        data['BTC'] = btc_balance
        data['CAD'] = cad_balance

        return data
        
    def _get_orderbook_from_api_req(self, verify=True):
        return self.req('get', '/order_book', no_auth=True, verify=verify)
    
    def place_order_req(self, mode, volume, price, order_type=order_types.LIMIT_ORDER):
        payload = {
            'amount': '%.8f' % volume.amount,
            'price': '%.2f' % price.amount,
        }

        if mode == Consts.BID:
            return self.req('post', '/buy', data=payload)
        elif mode == Consts.ASK:
            return self.req('post', '/sell', data=payload)
        else:
            raise ValueError('Mode must be either bid_string or the ask_string.')

    def place_order_resp(self, req):
        response = self.resp(req)

        try:
            return {'success': True, 'order_id': unicode(response['id'])}
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'response does not contain an order_id. Response: %s' % response,
            )
    
    def get_open_orders_req(self):
        return self.req('post', '/open_orders')
        
    def get_open_orders_resp(self, req):
        raw_open_orders = self.resp(req)
        raw_open_orders = raw_open_orders if raw_open_orders else []
        open_orders = []

        for raw_order in raw_open_orders:
            mode = self._order_mode_to_const(str(raw_order['type']))

            order = {
                'mode': mode,
                'id': str(raw_order['id']),
                'price': Money(raw_order['price'], 'CAD'),
                'volume_remaining': Money(raw_order['amount'], 'BTC'),
            }

            open_orders.append(order)

        return open_orders

    def get_order_details(self, order_id):
        req = self.get_order_details_req(order_id)
        return self.get_order_details_resp(req)
    
    def get_order_details_req(self, order_id):
        payload = {
            'id': unicode(order_id),
        }

        return self.req('post', '/lookup_order', data=payload)
    
    def get_order_details_resp(self, req):
        order = self.resp(req)

        try:
            order = order[0]
        except KeyError as e:
            ExchangeAPIErrorException(self, 'malformed response: %s' % order)
            
        trades = self.trades_for_order(order['id'])
        time_created = self._datetime_to_timestamp(order['created'])
        mode = self._order_mode_to_const(str(order['type']))

        result = {
            'time_created': time_created,
            'type': mode,
            'btc_total': sum([t['btc'] for t in trades], Money('0', 'BTC')),
            'fiat_total': sum([t['fiat'] for t in trades], Money('0', self.currency)),
            'trades': trades,
        }

        return result
    
    def get_multi_order_details_req(self, order_ids):
        data = {}

        for oid in order_ids:
            data[oid] = self.get_order_details_req(oid)

        return data
    
    def get_multi_order_details_resp(self, reqs):
        data = {}

        for oid in reqs:
            data[oid] = self.get_order_details_resp(reqs[oid])

        return data

    def get_ticker_req(self, verify=True):
        return self.req('get', '/ticker', no_auth=True, verify=verify)
    
    def get_ticker_resp(self, req):
        response = self.resp(req)

        return {
            'high': Money(response['high'], 'CAD'),
            'low': Money(response['low'], 'CAD'),
            'last': Money(response['last'], 'CAD'),
            'volume': Money(response['volume'], 'BTC'),
        }
    
    def cancel_order_req(self, order_id):
        payload = {
            'id': order_id,
        }

        return self.req('post', '/cancel_order', data=payload)
    
    def cancel_order_resp(self, req):
        response = self.resp(req)
        return {'success': True}

    def withdraw_crypto_req(self, address, volume):
        if not isinstance(address, basestring):
            raise TypeError('Withdrawal address must be a string')

        if not isinstance(volume, Money) or volume.currency != self.volume_currency:
            raise TypeError('Withdrawal volume must be in %s' % self.volume_currency)

        payload = {
            'amount': volume.amount,
            'address': address,
        }

        return self.req('post', '/bitcoin_withdrawal', data=payload)

    def withdraw_crypto_resp(self, req):
        response = self.resp(req)
        return {'success': True}

    def get_order_audit_data(self, skip_recent=0):
        """
        Returns an OrderedDict of order ids mapped to their filled volume (only include 
        orders that have some trades)
        """        
        orders = OrderedDict()
        trades_to_audit = self.trades()
        
        # Trades from the same order aren't guaranteed to be next to each other, so we
        # need to group them.
        trades_to_audit.sort(key=lambda t:(t['time']), reverse=True)

        for trade in trades_to_audit:
            order_id = str(trade['order_id'])
            try:
                orders[order_id] += abs(trade['btc'])
            except KeyError:
                orders[order_id] = abs(trade['btc'])

        # Remove the oldest 2 orders, because its trades might be wrapped around a page 
        # gap this would give us an innacurate volume_filled number. We need to remove
        # 2 because there could be an ask and a bid
        try:
            orders.popitem()
            orders.popitem()
        except KeyError:
            pass

        return orders




"""
This scripts creates a username/password credential in the dashboard database. We do
this through the command line to ensure that plaintext passwords are not stored in the
database.

Usage: gryphon-execute script create_dashboard_user [--execute]
"""

import getpass
import termcolor as tc
import os

from gryphon.lib import session
from gryphon.dashboards.models.user import User
from gryphon.dashboards.models.columns.password_column import Password


WARNING_MESSAGE = """
You are about to create a database user for the gryphon dashboard server.
This user will be able to access all of your dashboards, view trading returns, \
download ledger information, and generally know nearly everything about your trading \
business.
Are you sure you want to continue?"""

WARNING_PROMPT = """('y' to continue creating a user)):"""

EXIT_MESSAGE = """Ok. Exiting with no actions taken."""

CONTINUE_MESSAGE = """Ok, let's continue"""

ENTER_USERNAME_MESSAGE = """Enter your desired username: """

ENTER_PASSWORD_MESSAGE = """Enter your desired password: """

SUCCESS_NO_EXECUTE_MESSAGE = """\
Successfully created user object, but not committing because you did not add the \
--execute flag at the command line.
"""

SUCCESS_MESSAGE = """\
Success! You should be able to log in with the new credentials now.'\
"""


def main(script_arguments, execute):
    print tc.colored(WARNING_MESSAGE, 'red')
    informed_consent = raw_input(WARNING_PROMPT)

    if informed_consent != 'y':
        print EXIT_MESSAGE
        return
    else:
        print CONTINUE_MESSAGE

    dashboard_db = session.get_a_dashboard_db_mysql_session()

    username = raw_input(ENTER_USERNAME_MESSAGE)
    password_text = getpass.getpass(ENTER_PASSWORD_MESSAGE)

    password = Password(plain=password_text)
    
    user = User(username, password)

    if execute is True:
        dashboard_db.add(user)
        dashboard_db.commit()

        print tc.colored(SUCCESS_MESSAGE, 'green')
    else:
        print SUCCESS_NO_EXECUTE_MESSAGE
     



"""
"""

from cdecimal import Decimal

BUILTIN_STRAT_CONFIGS = {
    'manual': {
        'display_name': 'Manual',
        'strategy_actor': 'MULTI',
        'price_currency': 'USD',
        'volume_currency': 'BTC',
        'base_point_radius': 10,
        'graph_volume_threshold': Decimal('0.00001'),
        'position_graph_max': Decimal('100'),
        'position_graph_min': Decimal('-100'),
    },
    'multiexchange_linear': {
        'display_name': 'Multiexchange Linear BTC',
        'strategy_actor': 'MULTIEXCHANGE_LINEAR',
        'price_currency': 'USD',
        'volume_currency': 'BTC',
        'base_point_radius': 10,
        'graph_volume_threshold': Decimal('0.00001'),
        'position_graph_max': Decimal('2'),
        'position_graph_min': Decimal('-2'),
    },
    'simple_mm': {
        'display_name': 'Simple Market Making',
        'strategy_actor': 'SIMPLE_MM',
        'price_currency': 'USD',
        'volume_currency': 'BTC',
        'base_point_radius': 10,
        'graph_volume_threshold': Decimal('0.00001'),
        'position_graph_max': Decimal('2'),
        'position_graph_min': Decimal('-2'),
    },
    'simple_arb': {
        'display_name': 'Simple Arbitrage',
        'strategy_actor': 'SIMPLE_ARB',
        'price_currency': 'USD',
        'volume_currency': 'BTC',
        'base_point_radius': 10,
        'graph_volume_threshold': Decimal('0.00001'),
        'position_graph_max': Decimal('10'),
        'position_graph_min': Decimal('-10'),
    },
}





"""
"""
import pyximport; pyximport.install()
import os
import time
import unittest

from gryphon.fury.utils import audit
from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange
from gryphon.fury.harness.exchange_coordinator import ExchangeCoordinator
from gryphon.lib.session import get_a_mysql_session
from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange import order_types
from gryphon.lib.models.order import Order


class TestBitstampBTCUSDExchangeCoordinator(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestBitstampBTCUSDExchangeCoordinator, self).__init__(*args, **kwargs)

        self.db = get_a_mysql_session(creds=os.environ['TEST_DB_CRED'])
        self.exchange = ExchangeCoordinator(BitstampBTCUSDExchange(), self.db)

        self.order1_price_amount = '6'
        self.order2_price_amount = '7'

        self.order_volume = Money('1', self.exchange.volume_currency)
        self.order1_price = Money(self.order1_price_amount, self.exchange.currency)
        self.order2_price = Money(self.order2_price_amount, self.exchange.currency)

        self.sleep_time = 1

    def setUp(self):
        # Make sure there's no open orders on the exchange.
        assert self.exchange.get_open_orders() == []

        # Cancel any orders in the db that were left over from previous tests.
        self._db_cancel_open_orders()

    def tearDown(self):
        time.sleep(self.sleep_time)

        self.exchange.exchange_wrapper.cancel_all_open_orders()

        self._db_cancel_open_orders()

    # Tests. #

    def test_accounting_cancelled_order(self):
        """
        Create an order the ExchangeCoordinator::place_order, then cancel it on the
        exchange and run accounting. Verify that it gets cancelled in the db and there
        are no effects to the position of the exchange account in our db.
        """

        exchange_account = self.exchange.exchange_account_db_object(self.db)
        initial_position = exchange_account.position.copy()

        result = self._place_simple_order_1()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 
       
        self.exchange.cancel_order(order1_exchange_order_id)

        time.sleep(2)
 
        open_orders = self.exchange.get_open_orders()
        eaten_order_ids, current_orders = self.exchange._get_current_orders(open_orders)
        self.exchange._run_accounting(eaten_order_ids, current_orders)

        # TODO: Run some asserts here that verify the thing was done.
        db_order = self.exchange._get_orders_by_order_ids([order1_exchange_order_id])[0]
        new_exchange_account = self.exchange.exchange_account_db_object(self.db)
        new_position = new_exchange_account.position.copy()

        assert db_order.status == Order.CANCELLED
        assert initial_position == new_position

    def test_accounting_untouched_open_order(self):
        """
        Test that if we place an order on the exchange and leave it there,
        run_accounting won't mess with it in our db.
        """

        exchange_account = self.exchange.exchange_account_db_object(self.db)
        initial_position = exchange_account.position.copy()

        result = self._place_simple_order_1()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 
       
        time.sleep(2)
 
        open_orders = self.exchange.get_open_orders()
        eaten_order_ids, current_orders = self.exchange._get_current_orders(open_orders)

        assert len(eaten_order_ids) == 0
        assert len(current_orders) == 1
        assert order1_exchange_order_id == current_orders[0]['id']

        self.exchange._run_accounting(eaten_order_ids, current_orders)

        db_order = self.exchange._get_orders_by_order_ids([order1_exchange_order_id])[0]
        new_exchange_account = self.exchange.exchange_account_db_object(self.db)
        new_position = new_exchange_account.position.copy()

        assert db_order.status == Order.OPEN
        assert initial_position == new_position

    def test_accounting_tick_flow(self):
        """
        Test that we can just get through the accounting flow code-path without issue.
        """
        open_orders = self.exchange.get_open_orders()

        eaten_order_ids, current_orders = self.exchange._get_current_orders(open_orders)

        self.exchange._run_accounting(eaten_order_ids, current_orders)

    # Accounting tests TODO:
    #   - a partially filled order that is now cancelled
    #   - a fully filled order
    #   - a partially filled order that is still on the books

    def test_handle_unexpected_orders_1(self):
        """
        Case 1: skip the ExchangeCoordinator object to directly place an order on the
        exchange and make sure that we raise the appropriate exception.
        """
        result = self.exchange.exchange_wrapper.place_order(
            mode=Consts.BID,
            volume=self.order_volume,
            price=self.order1_price,
            order_type=order_types.LIMIT_ORDER,
        )

        exchange_open_orders = self.exchange.get_open_orders()

        # With no db representation of this order, we should raise an exception.
        self.assertRaisesRegexp(
            audit.AuditException,
            'Unexpected Orders \(not in db\):',
            self.exchange._get_current_orders,
            exchange_open_orders,
        )

    def test_handle_unexpected_orders_2(self):
        """
        Case 2: place an order through ExchangeCoordinator, manipulate it to a FILLED
        state in the database, and make sure we raise the appropriate exception.
        """

        result = self._place_simple_order_1()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 

        # Account the order as 'filled'
        db_order = self.exchange._get_orders_by_order_ids([order1_exchange_order_id])[0]
        db_order.status = Order.FILLED
        self.db.add(db_order)
        self.db.commit()

        # Double check it got in there.
        db_order = self.exchange._get_orders_by_order_ids([order1_exchange_order_id])[0]
        assert db_order.status == Order.FILLED

        # Verify thet get_current_orders raises an exception.
        exchange_open_orders = self.exchange.get_open_orders()

        self.assertRaisesRegexp(
            audit.AuditException,
            'Unexpected Orders \(shouldn\'t be in open state on exchange\):',
            self.exchange._get_current_orders,
            exchange_open_orders,
        )

        # Ugly cleanup.
        db_order = self.exchange._get_orders_by_order_ids([order1_exchange_order_id])[0]
        db_order.status = Order.OPEN
        self.db.add(db_order)
        self.db.commit()

    def test_handle_unexpected_orders_3(self):
        """
        Case 3: place an order on the exchange and cancel it in our database. Assert
        That after get_current_orders the order is open in our database again.
        """

        result = self._place_simple_order_1()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 

        db_order = self.exchange._get_orders_by_order_ids([order1_exchange_order_id])[0]
        db_order.status = Order.CANCELLED
        self.db.add(db_order)
        self.db.commit()

        db_order = self.exchange._get_orders_by_order_ids([order1_exchange_order_id])[0]
        assert db_order.status == Order.CANCELLED

        exchange_open_orders = self.exchange.get_open_orders()
        self.exchange._get_current_orders(exchange_open_orders)
        self.db.commit()  # Have to force commit here since we're not in a tick.
        
        # Since the order was cancelled in the database, but open on the exchange,
        # handle_unexpected should have triggered re-cancelling logic, which just
        # assumes a previous request failed and tries again.
        db_order = self.exchange._get_orders_by_order_ids([order1_exchange_order_id])[0]

        assert db_order.status == Order.OPEN

    def test_get_current_orders(self):
        """
        I'm pretty sure the thing to do here is to make two orders, cancel one, then
        run _get_current, and make sure the cancelled order is in eaten_order_ids, and
        the open order is in current_orders.

        You could also set up a situation where this tests the handle_unexpected case.
        """

        result = self._place_simple_order_1()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 

        result = self._place_simple_order_2()

        assert result['success'] is True
        order2_exchange_order_id = result['order_id'] 

        self.exchange.cancel_order(order1_exchange_order_id)

        exchange_open_orders = self.exchange.get_open_orders()

        eaten_order_ids, current_orders = self.exchange._get_current_orders(
            exchange_open_orders,
        )

        assert len(current_orders) == 1
        assert len(eaten_order_ids) == 1
        assert order1_exchange_order_id in eaten_order_ids
        assert current_orders[0]['id'] == order2_exchange_order_id

    def test_db_open_orders(self):
        """
        Place on order through ExchangeCoordinator::place_order and verify that it's in
        the response of _get_db_open_orders.
        """

        result = self._place_simple_order_1()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 

        db_open_orders = self.exchange._get_db_open_orders()

        assert len(db_open_orders) == 1
        assert db_open_orders[0].exchange_order_id == order1_exchange_order_id

    def test_get_orders_by_ids(self):
        """
        Place on order through ExchangeCoordinator::place_order and verify that it's in
        the response of ExchangeCoordinator::_get_orders_by_order_ids.
        """

        result = self._place_simple_order_1()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 

        result = self._place_simple_order_2()

        assert result['success'] is True
        order2_exchange_order_id = result['order_id'] 

        orders = self.exchange._get_orders_by_order_ids([
            order1_exchange_order_id,
            order2_exchange_order_id,
        ])

        db_oids = [o.exchange_order_id for o in orders]

        assert len(orders) == 2
        assert order1_exchange_order_id in db_oids
        assert order2_exchange_order_id in db_oids

    def test_order_placement(self):
        """
        Test the place_order function.
        """

        result = self._place_simple_order_1()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 

        db_order = self.db.query(Order)\
            .filter(Order.exchange_order_id == order1_exchange_order_id)\
            .first()

        assert db_order is not None
        assert db_order.exchange_order_id == order1_exchange_order_id
        assert db_order.status == 'OPEN'
        assert db_order.volume == self.order_volume
        assert db_order.price == self.order1_price

    # Helper functions. #

    def _place_simple_order_1(self):
        """
        Simple helper function to place one common order on the exchange.
        """
        result = self.exchange.place_order(
            mode=Consts.BID,
            volume=self.order_volume,
            price=self.order1_price,
            order_type=order_types.LIMIT_ORDER,
        )

        return result

    def _place_simple_order_2(self):
        """
        A companion to _1.
        """
        result = self.exchange.place_order(
            mode=Consts.BID,
            volume=self.order_volume,
            price=self.order2_price,
            order_type=order_types.LIMIT_ORDER,
        )

        return result

    def _db_cancel_open_orders(self):
        self.db.query(Order)\
            .filter(Order._exchange_name == self.exchange.name)\
            .filter(Order.status == Order.OPEN)\
            .update({Order.status: Order.CANCELLED})

        self.db.commit()



# -*- coding: utf-8 -*-

from gryphon.lib.logger import get_logger
logger = get_logger(__name__)
import requests
import time

class RetryRequest(object):
    
    @staticmethod
    def get(url, retry_attempts=0, **kwargs):
         RetryRequest._req('GET', url, retry_attempts, **kwargs)
         
    @staticmethod
    def post(url, retry_attempts=0, **kwargs):
        RetryRequest._req('POST', url, retry_attempts, **kwargs)
                
    @staticmethod
    def _req(http_method, url, retry_attempts=0, **kwargs):
        attempts = 0
        while True:
            try:
                if http_method == 'POST':
                    r = requests.post(url, **kwargs)
                elif http_method == 'GET':
                    r = requests.get(url, **kwargs)
                else:
                    raise NotImplementedError('HTTP METHOD does not exist')
                return r
            except Exception as e:
                attempts += 1
                if attempts > retry_attempts:
                    raise e
                else:
                    logger.info(u'[RETRY REQUEST] - This time failed.  Trying Again.')
                    time.sleep(0.1)



"""
"""
import pyximport; pyximport.install()
import os

from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange
from gryphon.fury.harness.exchange_coordinator import ExchangeCoordinator
from gryphon.tests.logic.exchange_coordinator.public_methods import ExchangePublicMethodsTests
from gryphon.tests.logic.exchange_coordinator.auth_methods import ExchangeAuthMethodsTests
from gryphon.lib.session import get_a_mysql_session
from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange import order_types
from gryphon.lib.models.order import Order


class TestKrakentExchangeCoordinator():
    def __init__(self):
        self.order1_price_amount = '10'
        self.order2_price_amount = '11'
        self.sleep_time = 1

    def setUp(self):
        self.db = get_a_mysql_session(creds=os.environ['TEST_DB_CRED'])
        self.exchange = ExchangeCoordinator(KrakenBTCEURExchange(), self.db)

    def tearDown(self):
        time.sleep(5)
        self.exchange.exchange_wrapper.cancel_all_open_orders()

    def test_order_placement(self):
        order_volume = Money('1', self.exchange.volume_currency)
        order1_price = Money(self.order1_price_amount, self.exchange.currency)
        order2_price = Money(self.order2_price_amount, self.exchange.currency)

        # Test the place_order function.

        result = self.exchange.place_order(
            mode=Consts.BID,
            volume=order_volume,
            price=order1_price,
            order_type=order_types.LIMIT_ORDER,
        )

        self.db.commit()

        assert result['success'] is True
        order1_exchange_order_id = result['order_id'] 

        db_order = self.db.query(Order)\
            .filter(Order.exchange_order_id == order1_exchange_order_id)\
            .first()

        assert db_order is not None
        assert db_order.exchange_order_id == order1_exchange_order_id
        assert db_order.status == 'OPEN'
        assert db_order.volume == order_volume
        assert db_order.price ==  order1_price








"""
These functions handle getting exchange volume numbers from the three different volume
data sources we have. These sources, listed in order of our preference, are:
i) Daily exchange ticker entries saved in ourd database
ii) Exchange trade lists saved in our database
iii) The Bitcoinwisdom API

The main interface of this library is get_single_exchange_volume_in_period and
get_total_exchange_volume_in_period. These two functions  decide which source to use
for a given time period and route the call the other functions in this file.
"""

from datetime import datetime, timedelta

from delorean import Delorean
from sqlalchemy import distinct, func

from gryphon.lib.bitcoinwisdom import BitcoinWisdom
from gryphon.lib.exchange.exchange_factory import ALL_EXCHANGE_KEYS
from gryphon.lib.models.emeraldhavoc.exchange_volume import ExchangeVolume
from gryphon.lib.models.emeraldhavoc.trade import Trade as EHTrade
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse

# this is a hack for now because we don't have consistent data
# sources for the other exchanges, and I don't want to dirty
# gryphon.lib's commit history with something like this.
exchanges_with_volume_sources = [
    'cavirtex', 
    'kraken', 
    'bitstamp', 
    'bitfinex', 
    'vaultofsatoshi',
]

EARLIEST_TICKER_ENTRY = parse('2015-11-13 19:30').datetime
EARLIEST_TRADE_ENTRY = parse('2015-11-25 23:59').datetime


# Functions for getting a single exchange's volume

def get_single_exchange_volume_in_period(tc_db, exchange_name, start_time, end_time):
    if is_past_single_day(start_time, end_time) and start_time > EARLIEST_TICKER_ENTRY:
        return get_single_exchange_volume_on_day_from_db_tickers(
            tc_db,
            exchange_name,
            start_time,
        )
    elif start_time > EARLIEST_TRADE_ENTRY:
        return get_single_exchange_volume_in_period_from_db_trades(
            tc_db,
            exchange_name,
            start_time,
            end_time,
        )
    elif exchange_name in exchanges_with_volume_sources:
        return get_single_exchange_volume_from_bitcoinwisdom(
            exchange_name,
            start_time,
            end_time,
        )
    else:
        return None


def get_single_exchange_volume_from_bitcoinwisdom(exchange_name, start_time, end_time):
    bw = BitcoinWisdom(exchange=exchange_name.lower())

    return bw.volume_in_period(
        start_time,
        end_time,
    )


def get_single_exchange_volume_on_day_from_db_tickers(tc_db, exchange_name, day_start):
    """
    Get the exchange volume for this exchange on a particular day from our saved
    ticker data.
    """

    # This is slightly counterintuitive. The volume for day x is in the first saved
    # ticker on day x + 1. So we look for that ticker.
    day_end = day_start + timedelta(days=1)
    five_minutes_later = day_end + timedelta(minutes=5)

    exchange_volume_obj = tc_db.query(ExchangeVolume)\
        .filter(ExchangeVolume.timestamp > day_end)\
        .filter(ExchangeVolume.timestamp < five_minutes_later)\
        .filter(ExchangeVolume.exchange == exchange_name)\
        .first()

    if not exchange_volume_obj:
        return None
    else:
        return exchange_volume_obj.exchange_volume


def get_single_exchange_volume_in_period_from_db_trades(tc_db, exchange_name, start_time, end_time):
    """
    Get the exchange volume for this exchange in this period from our saved version
    of the trade history.
    """

    total_volume = tc_db.query(func.sum(EHTrade._volume))\
        .filter(EHTrade.timestamp >= start_time)\
        .filter(EHTrade.timestamp < end_time)\
        .filter(EHTrade.exchange == exchange_name)\
        .scalar()

    return Money(total_volume, 'BTC')


# Functions for getting all integrated exchanges' volume in a period.

def get_total_exchange_volume_in_period(tc_db, start_time, end_time):
    if is_past_single_day(start_time, end_time) and start_time > EARLIEST_TICKER_ENTRY:
        return get_total_exchange_volume_on_day_from_db_tickers(tc_db, start_time)
    elif start_time > EARLIEST_TRADE_ENTRY:
        return get_total_exchange_volume_in_period_from_db_trades(
            tc_db,
            start_time,
            end_time,
        )
    else:
        return get_total_exchange_volume_from_bitcoinwisdom(
            start_time,
            end_time,
        )


def get_total_exchange_volume_on_day_from_db_tickers(tc_db, day_start):
    total_exchange_volume = Money(0, 'BTC')
    included_exchanges = []

    for exchange_name in ALL_EXCHANGE_KEYS:
        single_exchange_volume = get_single_exchange_volume_on_day_from_db_tickers(
            tc_db,
            exchange_name,
            day_start,
        )

        if single_exchange_volume:
            included_exchanges.append(exchange_name)
            total_exchange_volume += single_exchange_volume

    return total_exchange_volume, included_exchanges


def get_total_exchange_volume_from_bitcoinwisdom(start_time, end_time):
    total_exchange_volume = Money(0, 'BTC')
    included_exchanges = []

    for exchange_name in exchanges_with_volume_sources:
        single_exchange_volume = get_single_exchange_volume_from_bitcoinwisdom(
            exchange_name,
            start_time, 
            end_time,
        )

        if single_exchange_volume:
            included_exchanges.append(exchange_name)
            total_exchange_volume += single_exchange_volume

    return total_exchange_volume, included_exchanges


def get_total_exchange_volume_in_period_from_db_trades(tc_db, start_time, end_time):
    """
    Get the exchange volume for this exchange in this period from our saved version
    of the trade history.
    """

    # Watch this query for performance.
    results = tc_db.query(
            EHTrade.exchange,
            func.sum(EHTrade._volume),
        )\
        .filter(EHTrade.timestamp >= start_time)\
        .filter(EHTrade.timestamp < end_time)\
        .group_by(EHTrade.exchange)\
        .all()

    included_exchanges = [e[0].lower() for e in results]
    total_volume = sum([e[1] for e in results])

    return Money(total_volume, 'BTC'), included_exchanges


def get_hourly_total_exchange_volume_in_period(tc_db, start_time, end_time):
    return get_hourly_total_exchange_volume_in_period_from_db_trades(
        tc_db,
        start_time,
        end_time,
    )

def get_hourly_total_exchange_volume_in_period_from_db_trades(tc_db, start_time, end_time):
    """
    Get the exchange volume for this exchange in this period from our saved version
    of the trade history.
    """

    # Watch this query for performance.
    results = tc_db.query(
            func.hour(EHTrade.timestamp),
            func.sum(EHTrade._volume),
        )\
        .filter(EHTrade.timestamp >= start_time)\
        .filter(EHTrade.timestamp < end_time)\
        .group_by(func.hour(EHTrade.timestamp))\
        .all()

    formatted_results = []

    for row in results:
        hour = row[0]
        timestamp = Delorean(start_time, 'UTC').next_hour(hour).datetime
        volume = Money(row[1], 'BTC')

        formatted_results.append([
            timestamp,
            volume,
        ])

    formatted_results = sorted(formatted_results, key=lambda r: r[0])

    return formatted_results


def get_hourly_single_exchange_volume_in_period(tc_db, exchange_name, start_time, end_time):
    """
    We technically do have historical trade data now, but it's not proven yet so
    we're ignoring it until it is.
    """
    return get_hourly_single_exchange_volume_in_period_from_db_trades(
        tc_db,
        exchange_name,
        start_time,
        end_time,
    )


def get_hourly_single_exchange_volume_in_period_from_db_trades(tc_db, exchange_name, start_time, end_time):
    """
    Get the exchange volume for this exchange in this period from our saved version
    of the trade history.
    """

    # Watch this query for performance.
    results = tc_db.query(
            func.hour(EHTrade.timestamp),
            func.sum(EHTrade._volume),
        )\
        .filter(EHTrade.timestamp >= start_time)\
        .filter(EHTrade.timestamp < end_time)\
        .filter(EHTrade.exchange == exchange_name)\
        .group_by(func.hour(EHTrade.timestamp))\
        .all()

    formatted_results = []

    for row in results:
        hour = row[0]
        timestamp = Delorean(start_time, 'UTC').next_hour(hour).datetime
        volume = Money(row[1], 'BTC')

        formatted_results.append([
            timestamp,
            volume,
        ])

    formatted_results = sorted(formatted_results, key=lambda r: r[0])

    return formatted_results


def is_past_single_day(start_time, end_time):
    return (start_time.hour == 0) and (end_time - start_time == timedelta(days=1)) and end_time < Delorean().datetime





from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller


class PoloniexETHBTCOrderbook(OrderbookPoller):
    def __init__(self):
        self.exchange_name = u'POLONIEX_ETH_BTC'
        self.url = 'https://poloniex.com/public?command=returnOrderBook&currencyPair=BTC_ETH&depth=10000'  # Unclear what depth param we should be using.
        self.poll_time = 5

    # API response format:
    # {
    #   "asks":[["0.09022887",1704.11453071]],
    #   "bids":[["0.09000011",3.74072284]],
    #   "isFrozen":"0","seq":477056211,
    # }
    def parse_order(self, order):
        return [order[0], order[1], '']




"""
This adds a single function on the handler that gets query string arguments 'start' and
'end' and returns them as dates. The format for the arguments should be %Y-%m-%d,
e.g. 2015-10-22.

Optional parameters default_start and default_end specify what values the function
should return if either argument does not exist in the query string.
"""

from delorean import Delorean

from gryphon.lib.time_parsing import parse


class StartAndEndTimeMixin():
    def get_start_time_and_end_time(self, default_start=None, default_end=None):
        start = self.get_argument('start', None)
        end = self.get_argument('end', None)

        start_time = None
        end_time = None

        if start:
            start_time = parse(start).datetime
        elif default_start:
            start_time = default_start
        else:
            start_time = Delorean().truncate('day').datetime

        if end:
            end_time = parse(end).datetime
        elif default_end:
            end_time = default_end
        else:
            end_time = Delorean(start_time, 'UTC').next_day(1).datetime

        return start_time, end_time



# -*- coding: utf-8 -*-
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata

def unicode_string(self):
    return unicode(self).encode('utf-8')

Base.__str__ == unicode_string   


# How to migrate a database

#   foreman run alembic revision --autogenerate -m "moved value to Text from String"
# This generates the change script
#
#   foreman run alembic upgrade head
# This Executes the latest change script.



from collections import defaultdict
import json

from autobahn.twisted.websocket import connectWS, WebSocketClientFactory
from cdecimal import Decimal
from twisted.internet import defer, reactor, ssl

import gryphon.data_service.consts as consts
import gryphon.data_service.util as util
from gryphon.data_service.websocket_client import EmeraldWebSocketClientProtocol
from gryphon.data_service.pollers.orderbook.websocket.websocket_orderbook_poller import WebsocketOrderbookPoller
from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


class CoinbaseOrderbookWebsocket(EmeraldWebSocketClientProtocol, WebsocketOrderbookPoller):
    def __init__(self):
        self.exchange_name = u'COINBASE_BTC_USD'
        self.product_id = 'BTC-USD'
        self.base_url = 'https://api.exchange.coinbase.com/products/%s/book?level=3'
        self.url = self.base_url % self.product_id
        self.ping_interval = 5

    def connect_to_websocket(self):
        logger.info('Connecting to websocket')

        factory = WebSocketClientFactory(
            'wss://ws-feed.exchange.coinbase.com',
            debug=False,
        )

        # This actually creates a new Instance of CoinbaseOrderbook.
        factory.protocol = type(self)

        if factory.isSecure:
            context_factor = ssl.ClientContextFactory()
        else:
            context_factor = None

        connectWS(factory, context_factor)

    def subscribe_to_websocket(self):
        logger.info('Subscribing to websocket')

        data = {
            'type': 'subscribe',
            'product_id': self.product_id,
        }

        self.sendMessage(json.dumps(data))

    # REQUEST POLLER FUNCTIONS

    def parse_response(self, resp_obj):
        """
        This function will parse the base orderbook from the http endpoint.

        {"timestamp": "1412095328", "bids": [["382.74", "4.85241530"],
        """
        logger.info('Fetching Base Orderbook')

        self.orderbook_sequence_number = resp_obj['sequence']

        if (self.orderbook_sequence_number > self.first_sequence_number
                and self.first_sequence_number != 0):

            self.orderbook = {
                'bids': self.parse_orders(resp_obj['bids']),
                'asks': self.parse_orders(resp_obj['asks'])
            }

            logger.info(
                'Established Orderbook, Sequence: %s' % self.orderbook_sequence_number,
            )
        else:
            # Get the orderbook again since it was too old.
            logger.debug('Orderbook is too out-of-date. Retrying')

            reactor.callLater(1, self.get_request)

    # WEBSOCKET CLIENT FUNCTIONS

    @defer.inlineCallbacks
    def onOpen(self):
        logger.info('Connected to websocket')

        reactor.callLater(self.ping_interval, self.keepalive)

        self.redis = yield util.setup_redis()
        binding_key = '%s.orderbook.tinker' % self.exchange_name.lower()
        self.producer = yield util.setup_producer(consts.ORDERBOOK_QUEUE, binding_key)

        # Reset everything.
        self.orderbook = ''
        self.orderbook_change_backlog = {}
        self.first_sequence_number = 0
        self.current_sequence_number = 0
        self.orderbook_sequence_number = 0
        self.received_orders = {}
        self.last_amqp_push = 0
        yield self.redis.set(self.orderbook_key, None)

        # Start fetching the base orderbook from self.url. The request poller will call
        # parse_response with the response.
        self.get_request()

        self.subscribe_to_websocket()

    def keepalive(self):
        """
        Sends a keepalive ping to the Coinbase Websocket every 5 seconds. 5s is chosen
        so that we avoid the 10s "Cached orderbook too old" errors on gryphon-fury.
        Without any ping, the connection gets closed after 60 seconds of no messages.
        """
        logger.debug('Ping')
        self.sendPing('keepalive')

        reactor.callLater(self.ping_interval, self.keepalive)

    def onPong(self, payload):
        """
        Catch the pong back from Coinbase, and know that we are still successfully
        connected.
        """
        # We had one day where the Coinbase fundamental value did not change all day.
        # We think this may have been caused by them Ponging our Pings, but not
        # actually sending any data. Since Coinbase is so critical to our system, we
        # are only re-publishing orderbooks when there is an actual update (not just a
        # pong).
        logger.debug("Pong")

    @defer.inlineCallbacks
    def onMessage(self, payload, isBinary):
        should_continue = yield self.check_should_continue()

        if not should_continue:
            yield self.redis.set(self.orderbook_key, None)
            return

        payload = json.loads(payload)

        # Deal with sequence numbers.
        sequence_number = payload['sequence']
        if not self.first_sequence_number:
            self.first_sequence_number = sequence_number

        if (sequence_number == (1 + self.current_sequence_number) or
                sequence_number == self.first_sequence_number):
            # Keep going - sequence number is good.
            self.current_sequence_number = sequence_number
        else:
            # Sequence number is bad restart.
            logger.info('Sequencing Numbers on Coinbase came in wrong. Restarting.')

            yield self.restart()

        if payload['type'] in ['received', 'done', 'change', 'open']:
            if self.orderbook and not self.orderbook_change_backlog:
                # NO Backlog of changes. apply the change.
                self.apply_change_to_orderbook(payload)

                if payload['type'] != 'received':
                    self.publish_orderbook()

                return
            elif self.orderbook and self.orderbook_change_backlog:
                logger.info('Working down the backlog')

                # Adding current update to backlog.
                self.orderbook_change_backlog[sequence_number] = payload

                self.apply_backlog()

                if payload['type'] != 'received':
                    self.publish_orderbook()

                return
            else:
                logger.debug('Base Orderbook Not Ready')
                self.orderbook_change_backlog[payload['sequence']] = payload

                return
        else:
            # Match orders.
            pass

    # HELPER FUNCTIONS

    def apply_backlog(self):
        orderbook_backlog_sequence_numbers = sorted(
            self.orderbook_change_backlog.keys(),
        )

        num_changes = len(orderbook_backlog_sequence_numbers)
        logger.info('Applying %s changes to Orderbook' % num_changes)

        for seq_num in orderbook_backlog_sequence_numbers:
            if seq_num > self.orderbook_sequence_number:
                self.apply_change_to_orderbook(
                    self.orderbook_change_backlog.pop(seq_num),
                )
            else:
                # this update is too old. get rid of it
                self.orderbook_change_backlog.pop(seq_num)

    def apply_change_to_orderbook(self, change):
        """
        {
              "type": "received",
              "sequence": 10,
              "order_id": "<order id>",
              "size": "0.00",
              "price": "0.00",
              "side": "buy"
        }
        {
              "type": "open",
              "sequence": 10,
              "order_id": "d50ec984-77a8-460a-b958-66f114b0de9b",
              "price": "200.2",
              "remaining_size": "1.00"
        }

        {
              "type": "done",
              "sequence": 10,
              "price": "200.2",
              "order_id": "d50ec984-77a8-460a-b958-66f114b0de9b",
              "reason": "filled" // canceled
        }

        {
              "type": "change",
              "sequence": 80,
              "order_id": "ac928c66-ca53-498f-9c13-a110027a60e8",
              "time": "2014-11-07T08:19:27.028459Z",
              "new_size": "5.23512",
              "old_size": "12.234412",
              "price": "400.23"
        }
        """

        side = 'bids' if change['side'] == 'buy' else 'asks'

        if change['type'] == 'received':
            # add to orderbook to the received orders.
            self.received_orders[change['order_id']] = change

        elif change['type'] == 'open':
            # Grab the order from the received orders, create a new order in the
            # orderbook.
            received_order = self.received_orders[change['order_id']]
            self.received_orders.pop(change['order_id'])

            change.update(received_order)

            self.orderbook[side][change['order_id']] = {
                'price': change['price'],
                'volume': change['remaining_size'],
            }

        elif change['type'] == 'done':
            if change['order_id'] in self.orderbook[side]:
                self.orderbook[side].pop(change['order_id'])
            else:
                # An order was closed before it hit the orderbook - match.
                pass

        elif change['type'] == 'change':
            if change['order_id'] in self.orderbook[side]:
                self.orderbook[side][change['order_id']]['volume'] = change['new_size']
            elif change['order_id'] in self.received_orders:
                self.received_orders[change['order_id']]['volume'] = change['new_size']
            else:
                logger.info(
                    'Coinbase - change message order not in orderbook or received orders',
                )

        else:
            pass

    def get_order_from_orderbook(self, order_id):
        if order_id in self.orderbook['bids']:
            return self.orderbook['bids'][order_id]
        elif order_id in self.orderbook['asks']:
            return self.orderbook['asks'][order_id]
        else:
            return None

    def parse_orders(self, orders):
        """Returns a price keyed orders"""
        orders_dict = {}

        for order in orders:
            orders_dict[order[2]] = {'price': order[0], 'volume': order[1]}

        return orders_dict

    def get_orderbook_to_publish(self):
        orderbook = self.orderbook
        fancy_orderbook = {'bids': defaultdict(Decimal), 'asks': defaultdict(Decimal)}

        for bid in orderbook['bids']:
            fancy_orderbook['bids'][[bid]['price']] += Decimal(bid['volume'])

        for ask in orderbook['asks']:
            fancy_orderbook['asks'][ask['price']] += Decimal(ask['volume'])

        sorted_bid_keys = sorted(
            fancy_orderbook['bids'].keys(),
            key=lambda (k): float(k),
            reverse=True,
        )

        sorted_ask_keys = sorted(
            fancy_orderbook['asks'].keys(),
            key=lambda (k): float(k),
        )

        bids = [[k, str(fancy_orderbook['bids'][k]), ''] for k in sorted_bid_keys]

        asks = [[k, str(fancy_orderbook['asks'][k]), ''] for k in sorted_ask_keys]

        return {
            'bids': bids,
            'asks': asks,
        }




# -*- coding: utf-8 -*-
import time
import os
import datetime
import requests
import hmac
import urllib
import hashlib
import base64
import json
import math
from cdecimal import *
from delorean import Delorean
from collections import OrderedDict

from gryphon.lib.money import Money
from gryphon.lib.models.datum import DatumRecorder
from exchange_order import Order
from gryphon.lib.exchange.consts import Consts
from base import *
from exceptions import *
from gryphon.lib.models.exchange import Balance

from gryphon.lib.logger import get_logger
logger = get_logger(__name__)

class VaultOfSatoshiExchange(Exchange):
    def __init__(self, session=None, currency=u"CAD", use_cached_orderbook=False):
        super(VaultOfSatoshiExchange, self).__init__(session)
        self.name = u'VAULTOFSATOSHI'
        self.friendly_name = u'Vault of Satoshi'
        self.base_url = 'https://api.vaultofsatoshi.com'
        self.currency = currency
        self.fee = Decimal("0") # assuming $99 unlimited account
        self.withdrawal_fee = Money("0.0005", "BTC")
        self.bid_string = "bid"
        self.ask_string = "ask"
        self.use_cached_orderbook = use_cached_orderbook



# -*- coding: utf-8 -*-
"""
This class is used to represent liabilities against the business or fund. It is used
extensively in gryphon.lib.assets to give a correct picture of the business's
financial position.

TODO:
  - Implement functions to calculate the current repayable value of a liability, be it
    a loan or a fund investment.
  - the current implementation of self.details, self.interest_rate,
    self.compounding_period, is really just a demonstraction. It should be re-
    implemented in the style of the balance column in the exchange account model.
"""
from datetime import datetime
import json
import uuid

from cdecimal import Decimal
from delorean import Delorean
from sqlalchemy import Column, Integer, Unicode, DateTime, UnicodeText, Numeric

from gryphon.lib.logger import get_logger
from gryphon.lib.models.base import Base
from gryphon.lib.money import Money

logger = get_logger(__name__)

metadata = Base.metadata


UNICODE_STRING_REP = """\
[LIABILITY: %s to %s of type %s started %s and repayed %s]\
"""

class Liability(Base):
    __tablename__ = 'liability'

    # Liability types.
    FIXED_INTEREST = 'FIXED_INTEREST'  # Fixed interest loans or similar.
    PERFORMANCE = 'PERFORMANCE'  # Fund investments.

    # Columns.
    # Meta-columns.
    liability_id = Column(Integer, primary_key=True)
    unique_id = Column(Unicode(64), nullable=False)
    time_created = Column(DateTime, nullable=False)

    # Data columns.
    _amount = Column('amount', Numeric(precision=24, scale=14))
    _amount_currency = Column('amount_currency', Unicode(3))
    liability_type = Column(Unicode(64))
    entity_name = Column(Unicode(128), nullable=False)
    time_started = Column(DateTime, nullable=True)
    time_repayed = Column(DateTime, nullable=True)
    _details = Column('details', UnicodeText(length=2**31))

    def __init__(self, amount, liability_type, entity_name, time_started=None, time_repayed=None, details=None):
        self.unique_id = unicode(uuid.uuid4().hex)
        self.time_created = datetime.utcnow()

        self.amount = amount
        self.liability_type = liability_type
        self.entity_name = entity_name
        self.time_started = time_started
        self.time_repayed = time_repayed
        self.details = details

    def __str__(self):
        return UNICODE_STRING_REP % (
            self.amount,
            self.entity_name,
            self.liability_type,
            self.time_started if self.time_started else '[NOT STARTED]',
            self.time_repayed if self.time_repayed else '[NOT REPAYED]',
        )

    def __repr__(self):
        return self.to_json()

    def to_json(self):
        return json.dumps({
            'liability_id': self.liability_id,
            'unique_id': self.unique_id,
            'time_created': unicode(self.time_created),
            'amount': self.amount,
            'liability_type': self.liability_type,
            'entity_name': self.entity_name,
            'time_started': unicode(self.time_started),
            'time_repayed': unicode(self.time_repayed),
            'details': self.details,
        }, ensure_ascii=False)

    @property
    def details(self):
        if self._details != 'null':
            return json.loads(self._details)
        else:
            return {}

    @details.setter
    def details(self, value):
        self._details = json.dumps(value, ensure_ascii=False)

    @property
    def amount(self):
        return Money(self._amount, self._amount_currency)

    @amount.setter
    def amount(self, value):
        self._amount = value.amount
        self._amount_currency = value.currency

    def start(self):
        self.time_started = Delorean().datetime

    def complete(self):
        self.time_repayed = Delorean().datetime

    @property
    def interest_rate(self):
        if 'interest_rate' in self.details:
            return Decimal(str(self.details['interest_rate']))
        else:
            return Decimal('0')

    @interest_rate.setter
    def interest_rate(self, value):
        """
        We might save some code (or duplication) if we used something like the
        JSONEncodedMoneyDict from the exchange model here, but this works for now, and
        this table is expected to be small so this shouldn't affect performance.
        """
        new_details = self.details
        new_details['interest_rate'] = float(value)
        self.details = new_details

    @property
    def compounding_period(self):
        """
        We track the compounding period of fixed-interest debts as an integer number
        of months.
        """
        if 'compounding_period' in self.details:
            return Decimal(str(self.details['compounding_period']))
        else:
            return Decimal('0')

    @compounding_period.setter
    def compounding_period(self, value):
        new_details = self.details
        new_details['compounding_period'] = float(value)
        self.details = new_details




import os
import random
import subprocess
import time

from delorean import Delorean
import pygerduty
from sqlalchemy.orm import joinedload

from gryphon.execution.lib.heartbeat import heartbeat
from gryphon.lib import hackernewsie
from gryphon.lib import redditor
from gryphon.lib import session
from gryphon.lib.exchange import exchange_factory
import gryphon.lib.gryphonfury.positions as positions
import gryphon.lib.gryphonfury.revenue as revenue_lib
from gryphon.lib.logger import get_logger
from gryphon.lib.models.datum import DatumRecorder
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money
from gryphon.lib.slacker import Slacker
from gryphon.lib.time_parsing import parse
from gryphon.lib.util.time import humanize_seconds

logger = get_logger(__name__)

TICK_SLEEP = 60
SHOEBOX_HEARTBEAT_KEY = 'SHOEBOX'


def run():
    db = session.get_a_trading_db_mysql_session()

    DatumRecorder().create(db=db)

    logger.info('Reporting for duty.')

    try:
        while True:
            current_time = Delorean().datetime

            try:
                update_tx_hashes(db)
            except Exception as e:
                logger.exception('Error while updating transaction hashes')

            # Checking the exact minute amounts works because the shoebox is ticking every 1m
            # This may not work 100% if the shoebox tick takes any noticable amount of time
            # to run. In that case we might tick at the end of xx:59 and the beginning of xx:01,
            # and these tasks wouldn't run. We'll keep an eye on it and may have to add some
            # persistance later.

            # every 5 minutes
            if current_time.minute % 5 == 0:
                try:
                    get_breaking_bitcoin_news()
                except Exception:
                    logger.exception('Error while getting breaking news')

            # every hour
            if current_time.minute == 0:
                money_moving()
                manual_btc_withdrawals(db)

            # end of day UTC
            if current_time.hour == 0 and current_time.minute == 0:
                notify_revenue(db)

            heartbeat(SHOEBOX_HEARTBEAT_KEY)

            session.commit_mysql_session(db)
            logger.info('Going to sleep for %s.' % humanize_seconds(TICK_SLEEP))
            time.sleep(TICK_SLEEP)
    finally:
        db.remove()


def get_breaking_bitcoin_news():
    reddit_breaking_news = redditor.get_reddit_breaking_news('bitcoin', upvote_threshold=500)

    hn_keyword_dict = {
        'bitcoin': 15,
        'bitstamp': 5,
        'bitfinex': 5,
        'cavirtex': 5,
        'quadriga': 5,
        'okcoin': 5,
        'coinbase': 5,
        'coinsetter': 5,
        'gemini': 5,
        'kraken': 5,
    }

    hn_breaking_news = hackernewsie.get_hn_breaking_news(hn_keyword_dict)

    for news in reddit_breaking_news:
        news['source'] = 'Reddit'

    for news in hn_breaking_news:
        news['source'] = 'HN'

    breaking_news = reddit_breaking_news + hn_breaking_news
    if breaking_news:
        message = 'Breaking News:\n'

        for news in breaking_news:
            message += '<%s|%s> (<%s|%s comments>)\n\n' % (news['url'], news['title'], news['comments_url'], news['source'])

        slacker = Slacker('#news', 'news')
        slacker.notify(message)


def update_tx_hashes(db):
    logger.info('Updating transaction hashes.')
    txs = in_progress_transactions(db)
    for tx in txs:
        tx.update_tx_hash()


def in_progress_transactions(db):
    txs = db.query(Transaction)\
        .filter_by(transaction_status='IN_TRANSIT')\
        .join(ExchangeData)\
        .order_by(Transaction.time_created)\
        .all()
    return txs


def load_devs():
    devs = [
        {
            'name': 'gareth',
            'starts_work': 11,
            'ends_work': 19,
            'timezone': 'US/Pacific',
        },
        {
            'name': 'trevor',
            'starts_work': 11,
            'ends_work': 19,
            'timezone': 'US/Pacific',
        },
        {
            'name': 'ross',
            'starts_work': 11,
            'ends_work': 19,
            'timezone': 'US/Pacific',
        },
    ]

    return devs


def dev_checkins():
    """Sends a checkin message to all the working devs at the top of the hour"""

    logger.info('Running dev checkins')

    # devs should get pinged after every hour of work
    # so if they are working from 10am to 5pm they should get pinged
    # at 11am, 12pm, 1pm, 2pm, 3pm, 4pm and 5pm
    devs = load_devs()

    for dev in devs:
        current_time = Delorean(timezone=dev['timezone'])

        if is_working(current_time, dev):
            message_dev_for_checkin(dev, current_time)


def dev_summaries():
    """Sends a summary request message to all the working devs at the end of each workday"""
    logger.info('Running dev summaries')

    # devs should get pinged after every hour of work
    # so if they are working from 10am to 5pm they should get pinged
    # at 11am, 12pm, 1pm, 2pm, 3pm, 4pm and 5pm
    devs = load_devs()

    for dev in devs:
        current_time = Delorean(timezone=dev['timezone'])

        if is_end_of_workday(current_time, dev):
            message_dev_for_summary(dev)


def is_end_of_workday(current_time, dev):
    current_hour = current_time.datetime.hour
    is_end_of_day = (current_hour == dev['ends_work'])
    return is_workday(current_time) and is_end_of_day


def is_workday(current_time):
    weekday = current_time.datetime.weekday()
    is_saturday = (weekday == 5)
    is_sunday = (weekday == 6)

    return (not is_saturday and not is_sunday)


def is_working(current_time, dev):
    """Is the dev working at the current time?"""

    current_hour = current_time.datetime.hour
    is_working_hours = (current_hour > dev['starts_work'] and current_hour <= dev['ends_work'])
    return is_workday(current_time) and is_working_hours


def message_dev_for_summary(dev):
    channel = '#general'
    message = 'Hey @%s, what did you get up to today?' % dev['name']
    emoji = fun_emoji()
    bot_name = fun_bot_name(emoji)
    slacker = Slacker(channel, bot_name, icon_emoji=emoji)
    slacker.notify(message)


def message_dev_for_checkin(dev, current_time):
    logger.info('pinging @%s' % dev['name'])

    channel = '@%s' % dev['name']
    message = 'Time for a quick checkin'
    emoji = clock_emoji(current_time)
    slacker = Slacker(channel, 'checkin', icon_emoji=emoji)
    slacker.notify(message)


def clock_emoji(time):
    hour = time.datetime.hour
    # 14 -> 2(pm)
    clock_hour = hour % 12

    # there's no 0-hour emoji
    if clock_hour == 0:
        clock_hour = 12

    return ':clock%s:' % clock_hour


def fun_emoji():
    fun_emoji_list = [
        ':snowman:',
        ':cat:',
        ':dog:',
        ':mouse:',
        ':hamster:',
        ':rabbit:',
        ':wolf:',
        ':frog:',
        ':tiger:',
        ':koala:',
        ':bear:',
        ':pig:',
        ':cow:',
        ':boar:',
        ':monkey:',
        ':horse:',
        ':snake:',
        ':bird:',
        ':baby_chick:',
        ':chicken:',
        ':penguin:',
        ':turtle:',
        ':bug:',
        ':honeybee:',
        ':ant:',
        ':beetle:',
        ':snail:',
        ':octopus:',
        ':tropical_fish:',
        ':fish:',
        ':whale:',
        ':whale2:',
        ':dolphin:',
        ':rat:',
        ':octocat:',
        ':squirrel:',
        ':ghost:',
    ]
    return random.choice(fun_emoji_list)


def fun_bot_name(emoji):
    # ':tropical_fish:' -> 'Tropical Fishbot'
    return ' '.join([w.capitalize() for w in emoji.replace(':', '').split('_')]) + 'bot'

MANUAL_BTC_EXCHANGES = {
    'CAVIRTEX': Money('40', 'BTC'),
    'COINSETTER': Money('40', 'BTC'),
}
BTC_TRANSFER_UNIT = Money('30', 'BTC')


def manual_btc_withdrawals(db):
    """
    Check the Manual BTC exchanges for being above their target. Ping a dev on hipchat if they are.
    """

    logger.info('Running manual BTC withdrawals')

    for name, target in MANUAL_BTC_EXCHANGES.iteritems():
        exchange_db = exchange_factory.make_exchange_data_from_key(name, db)
        if exchange_db.balance['BTC'] > target:
            withdrawal_amount = exchange_db.balance['BTC'] - target
            if withdrawal_amount > BTC_TRANSFER_UNIT:
                btc_withdrawal_notification(exchange_db, withdrawal_amount)


def btc_withdrawal_notification(exchange_db, amount):
    on_call_dev = get_on_call_dev()
    amount = amount.round_to_decimal_places(0)
    channel = '@%s' % on_call_dev
    message = 'Do a %d BTC withdrawal from %s' % (amount.amount, exchange_db.name)

    logger.info('Hipchat ping: %s' % message)

    slacker = Slacker(channel, 'withdrawals', icon_emoji=':moneybag:')
    slacker.notify(message)


def money_moving():
    current_time = Delorean().datetime

    morning_timeslot = parse('9am -0800').datetime  # when we get up in PST
    afternoon_timeslot = parse('3:30pm -0500').datetime  # 4pm EST is BMO's wire cutoff

    in_morning_timeslot = (current_time.hour == morning_timeslot.hour and
        current_time.minute == morning_timeslot.minute)

    in_afternoon_timeslot = (current_time.hour == afternoon_timeslot.hour and
        current_time.minute == afternoon_timeslot.minute)

    if in_morning_timeslot or in_afternoon_timeslot:
        on_call_dev = get_on_call_dev()
        channel = '@%s' % on_call_dev

        if in_morning_timeslot:
            message = 'Time for early morning money moving!'
        elif in_afternoon_timeslot:
            message = 'Time for afternoon money moving! (Wire cutoff is 4pm EST)'

        slacker = Slacker(channel, 'mover', icon_emoji=':moneybag:')
        slacker.notify(message)


def notify_revenue(db):
    # beginning of last UTC day
    start_time = Delorean().last_day().truncate('day').datetime
    end_time = Delorean().truncate('day').datetime

    # This revenue calculating logic takes about 10s to run
    open_position_offset = positions.fast_position(
        db,
        end_time=start_time,
    )

    open_position_trades = revenue_lib.open_position_trades(
        open_position_offset,
        db,
        start_time,
    )

    trades = db\
        .query(Trade)\
        .join(Order)\
        .filter(Order.actor == 'Multi')\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order'))\
        .order_by(Trade.time_created)\
        .all()

    matched_trades, _ = revenue_lib.split_trades(
        open_position_trades + trades,
    )

    __, revenue, __, __ = revenue_lib.profit_data(matched_trades, 'USD')

    slacker = Slacker('#general', 'revenue', icon_emoji=':moneybag:')
    slacker.notify('We made %s today' % revenue)


def get_on_call_dev():
    """ Returns the username (gareth, ross, trevor) of the current on-call dev """
    pager = pygerduty.PagerDuty('tinker', os.environ['PAGERDUTY_API_KEY'])

    escalation_policies = pager.escalation_policies.on_call()
    default_policy = [p for p in escalation_policies if p.name == 'Default'][0]

    on_call_users = default_policy.on_call
    on_call = [u for u in on_call_users if u.level == 1][0]

    name = on_call.user.name
    first_name = name.split()[0]
    username = first_name.lower()

    return username



# -*- coding: utf-8 -*-
import tornado.web

from admin_base import AdminBaseHandler


class HomeHandler(AdminBaseHandler):
    @tornado.web.authenticated
    def get(self):
        self.render_template('home.html', args={})



from operator import attrgetter

import gryphon.lib.gryphonfury.positions as positions
import gryphon.lib.gryphonfury.revenue as revenue_lib
import gryphon.dashboards.util.queries as queries


def get_trades_and_position_trades(db, start_time, end_time):
    """
    There is still a missing refactor in all our profit calculation code
    there's a bit of duplication in gryphon handler and I think the whole
    thing could still be simpler.
    """
    trades = queries.get_all_trades_for_period(
        db,
        start_time,
        end_time,
    )

    trades.sort(key=attrgetter('time_created'))

    # get system position up until the starting point of our graph
    open_position_offset = positions.fast_btc_position(
        db,
        end_time = start_time,
    )

    open_position_trades = revenue_lib.open_position_trades(
        open_position_offset, 
        db,
        start_time,
    )

    return trades, open_position_trades




# -*- coding: utf-8 -*-
"""
The Harness is the execution context we provide to Strategies for them to interact with
the outside world. This allows us to provide a level of robustness and simplicity of
interface that would be strictly impossible if strategies were to interact directly with
exchanges themselves.

Adding this level of abstraction is useful for other purposes, too. Sometimes, you'll
want your strategies to run in a context that isn't the real world, such as during
backtesting. For this, you can write another harness (so long as it conforms to the same
interface), and reimplement the interface functions in a different way.
"""

from sets import Set
import termcolor as tc
import time

from cdecimal import ROUND_UP, ROUND_DOWN
from delorean import Delorean
from sqlalchemy.orm import joinedload

from gryphon.execution.lib import auditing
from gryphon.execution.harness.exchange_coordinator import ExchangeCoordinator
from gryphon.lib import configuration as config_lib
from gryphon.lib.configurable_object import ConfigurableObject
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.exchange.retry import exchange_retry
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exceptions import CancelOrderNotFoundError
from gryphon.lib.logger import get_logger
from gryphon.lib.models.datum import DatumRecorder
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.money import Money
from gryphon.lib.session import commit_mysql_session
from gryphon.lib.util.profile import tick_profile

logger = get_logger(__name__)

ORDERBOOK_DELAY_SAMPLE_SIZE = 10


class Harness(ConfigurableObject):
    def __init__(self, db, configuration):
        self.db = db

        self.strategy = None

        # Configurables.
        self.execute = False
        self.emerald = False
        self.audit = False
        self.audit_tick = 100
        self.audit_types = []

        if configuration:
            self.configure(configuration)

        self.initialize_coordinators(configuration)

    def configure(self, configuration):
        self.init_configurable('execute', configuration['platform'])
        self.init_configurable('emerald', configuration['platform'])
        self.init_configurable('audit', configuration['platform'])
        self.init_configurable('audit_tick', configuration['platform'])
        self.init_configurable('audit_types', configuration['platform'])

        if type(self.audit_types) is str:
            if self.audit_types.lower() == 'all':
                self.audit_types = auditing.ALL_AUDITS
            else:
                self.audit_types = config_lib.parse_configurable_as_list(
                    self.audit_types,
                )

    def initialize_coordinators(self, configuration):
        """
        The harness has an exchange connection for every gryphon integrated exchange.
        Only those exchanges which have an initialized ledger and proper credentials
        will be 'tradable', or able to access authenticated endpoints. You can tell
        which exchanges are tradable by looking at harness.tradable_exchanges.
        """

        self.exchanges = []

        for exchange_name in exchange_factory.ALL_EXCHANGE_KEYS:
            api_wrapper_class = exchange_factory.get_api_wrapper_class_by_name(
                exchange_name,
            )

            exchange_coordinator = ExchangeCoordinator(
                api_wrapper_class(configuration=configuration),
                self.db,
                self,
            )

            setattr(self, exchange_name.lower(), exchange_coordinator)

            self.exchanges.append(exchange_coordinator)

        self.log('The following exchanges will be tradable: %s' % (
            [e.name for e in self.tradable_exchanges],
        ))

    def exchange_from_key(self, exchange_name):
        return self._exchange_dict[exchange_name.lower()]

    @property
    def _exchange_dict(self):
        return {e.name.lower(): e for e in self.exchanges}

    @property
    def tradable_exchanges(self):
        """
        A tradable exchange is one for which we have an initialized ledger in the db
        (a row in the Exchange table), and which has a working api connection.
        """
        return [e for e in self.exchanges if e.is_tradable]

    @property
    def non_tradable_exchanges(self):
        """
        A tradable exchange is one for which we have an initialized ledger in the db
        (a row in the Exchange table), and which has a working api connection.
        """
        return [e for e in self.exchanges if not e.is_tradable]

    @property
    def active_exchanges(self):
        """
        Active exchanges are those that have had trading activity since the last audit,
        which means we have to run consolidate_ledger() for them every tick.
        """
        return [e for e in self.exchanges if e.is_active]

    @property
    def auditable_exchanges(self):
        """
        Auditable exchanges are those that we'll run audits on, either because they've
        had trading activity on them or because the strategy listed them in
        target_exchanges. Only tradable exchanges are auditable.
        """
        return [
            e for e in self.tradable_exchanges
            if e.is_active or e.name in self.strategy.target_exchanges
        ]

    ## Tick code-path functions. ##

    def tick(self):
        """
        Consolidate the ledgers and the tick the strategy.
        """

        self.pre_tick_algo()

        current_orders = self.consolidate_ledgers()

        self.pre_tick_logging(current_orders)

        self.tick_algo(current_orders)

    @tick_profile
    def pre_tick_algo(self):
        self.strategy.pre_tick()

    @tick_profile
    def tick_algo(self, current_orders):
        self.strategy.tick(current_orders)

    def post_tick(self, tick_count):
        self.strategy.post_tick(tick_count)

    def sleep_time_to_next_tick(self):
        # Is tick a property of the strategy or of the platform...
        # It can be a property of the strategy and still a built-in argument.
        return self.strategy.tick_sleep

    def pre_tick_logging(self, current_orders):
        self.log_position()
        self.log_balances()
        self.log_orders(current_orders)

    def consolidate_ledgers(self):
        """
        Make sure that our ledger agrees with the exchange's state for each exchange
        that we've been making trades on.
        """

        current_orders = {}

        for exchange in self.active_exchanges:
            exchange_current_orders = exchange.consolidate_ledger()
            current_orders[exchange.name] = exchange_current_orders

        return current_orders

    ## Harness Interface Functions. ##

    def strategy_complete(self):
        return self.strategy.is_complete()

    ## Accounting and auditing functions. ##

    @exchange_retry()
    def wind_down(self):
        self.log('Winding Down')

        self.log('Cancelling All Open Orders')

        for exchange in self.auditable_exchanges:
            exchange.cancel_all_open_orders()

        time.sleep(10)

        self.log('Consolidating Ledgers')

        current_orders = self.consolidate_ledgers()

        # All orders should have been cancelled by now.
        assert all([len(orders) == 0 for orders in current_orders.values()])

    ##  Logging functions. ##

    def log(self, format_string, string_args=None, color=None, log_level='info'):
        """
        This function simply attaches the date and colours to a log message, and then
        dispatches to the appropriate logger function.
        """
        timestamp = unicode(Delorean().datetime.strftime('%d/%m/%y %H:%M:%S %Z'))

        result_string = u'[%s] (%s) %s' % (
            self.strategy.name if self.strategy else 'HARNESS_SETUP',
            timestamp,
            format_string,
        )

        if string_args != None:
            result_string = result_string % string_args

        if color:
            result_string = tc.colored(result_string, color)

        if log_level == 'info':
            logger.info(result_string)
        elif log_level == 'debug':
            logger.debug(result_string)
        elif log_level == 'error':
            logger.error(result_string)
        elif log_level == 'critical':
            logger.critical(result_string)

    def log_position(self):
        volume_position = self.strategy.position
        self.log('Open position: %s', volume_position, 'magenta')

    def log_balances(self):
        volume_currency = self.strategy.volume_currency

        self.log('Available balances:', color='magenta')

        for exchange in self.auditable_exchanges:
            volume_balance = exchange.exchange_account.balance[volume_currency]
            price_balance = exchange.exchange_account.balance[exchange.currency]

            self.log(
                '  %s: %s %s',
                (exchange.name, volume_balance, price_balance),
                'magenta',
            )

        # We currently only support lines-of-credit for Bitcoin.
        if self.strategy.volume_currency == 'BTC' and False:
            btc_credit_limit = self.strategy.exchange.btc_credit_limit

            if btc_credit_limit > 0:
                self.log(
                    'With Credit: %s',
                    (volume_balance + btc_credit_limit),
                    'magenta',
                )

    def log_orders(self, current_orders):
        """
        TODO: Let strategies overload this function somehow so they can give their own
        representation of the open order state (e.g. get our [asks][fv][bids] display
        back for market making strats.
        """

        self.log('Open Orders:', color='cyan')

        for exchange_name, exchange_orders in current_orders.items():
            self.log('  %s' % exchange_name, color='cyan')

            bids = [o for o in exchange_orders if o['mode'] == Consts.BID]
            asks = [o for o in exchange_orders if o['mode'] == Consts.ASK]
            bids.sort(key=lambda o: o['price'], reverse=True)
            asks.sort(key=lambda o: o['price'], reverse=True)

            for ask in asks:
                self.log(
                    '\tAsk : %.2f   %.4f',
                    (ask['price'].amount, ask['volume_remaining'].amount),
                    'cyan',
                )

            for bid in bids:
                self.log(
                    '\tBid : %.2f   %.4f',
                    (bid['price'].amount, bid['volume_remaining'].amount),
                    'cyan',
                )

    def log_trade(self, fiat_change, volume_change):
        """
        Log trades as we account for them. TODO: This function has recently been moved
        into ExchangeCoordinator, where it has access to the specific order information,
        so we could rewrite this function to print out more specific info than just
        the relative position change.
        """

        # No trades happened.
        if volume_change == Money('0', self.strategy.volume_currency):
            return

        price_per_unit_volume = abs(fiat_change) / abs(volume_change).amount

        if volume_change > 0:  # Bought tokens.
            self.log(
                'Bought %s at a price of %s',
                (abs(volume_change), abs(fiat_change)),
                'green',
            )
        elif volume_change < 0:  # Sold tokens.
            self.log(
                'Sold %s at a price of %s',
                (abs(volume_change), abs(fiat_change)),
                'green',
            )

    # Auditing functions.

    def full_audit(self, wind_down=True):
        if wind_down:
            self.wind_down()

        for exchange in self.auditable_exchanges:
            exchange.audit(self.audit_types)




import logging
import os

import selenium
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

from gryphon.lib.logger import get_logger
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money
from gryphon.lib.scrapers.base import Scraper
from gryphon.lib.time_parsing import parse

logger = get_logger(__name__)


class BMOScraper(Scraper):
    def __init__(self):
        self.is_ready = False
        self.account_data = []

    def load_transactions(self, account_num):
        """
        Loads the transactions page for each account, and adds them to self.account_data
        """
        self.load()

        logger.debug('transactions')

        for account_idx, account in enumerate(self.account_data):
            if account['account_number'] == account_num:
                # account_idx and account now match the account_num we passed in
                break

        # this is a bit fragile, but chances are if BMO changes their JS framework
        # they'll also change the page layout/markup and this whole scraper will break
        path = '/onlinebanking/OLB/fin/acc/adt/accountDetailsInit?mode=confirmation'
        js = "goto('%s',{inquiryAccountIndex:'%d'})" % (path, account_idx)
        self.driver.execute_script(js)

        transactions = self.get_transactions_from_page(account)

        return transactions

    def get_transactions_from_page(self, account):
        """
        Parses out records from a transactions page.

        Assumes the driver has already navigated to a transactions page.
        """
        logger.debug('get_transactions_from_page')

        account_num = account['account_number']
        account_currency = account['balance'].currency

        self.wait_for_text_on_page('Transaction History')
        self.wait_for_text_on_page('Balance Forward')

        # We need to look for the specific account number because the wait_for_text checks above
        # can match on the transactions page we were previously on
        logger.debug('waiting for account number to match %s' % account_num)
        WebDriverWait(self.driver, 10).until(EC.text_to_be_present_in_element((By.CSS_SELECTOR, '.bodyCopy .cardNumber'), account_num))

        transaction_selector = '#ccChequingTransactionTable tbody tr'
        transaction_els = self.driver.find_elements_by_css_selector(transaction_selector)
        transaction_els.pop(0) # drop the Balance Forward row

        transactions = []

        for el in transaction_els:
            date = el.find_element_by_css_selector('td:nth-child(1)').text
            code = el.find_element_by_css_selector('td:nth-child(2)').text
            description = el.find_element_by_css_selector('td:nth-child(3)').text
            raw_debit = el.find_element_by_css_selector('td:nth-child(4)').text
            raw_credit = el.find_element_by_css_selector('td:nth-child(5)').text
            raw_balance = el.find_element_by_css_selector('td:nth-child(6)').text

            if raw_debit and raw_credit:
                raise Exception('Only one of debit and credit fields should be present')
            elif not raw_debit and not raw_credit:
                raise Exception('One of debit and credit fields should be present')
            elif raw_debit:
                debit = self.string_to_money(raw_debit + ' ' + account_currency)
                amount = debit
                transaction_type = Transaction.WITHDRAWL
            elif raw_credit:
                credit = self.string_to_money(raw_credit + ' ' + account_currency)
                amount = credit
                transaction_type = Transaction.DEPOSIT

            balance = self.string_to_money(raw_balance + ' ' + account_currency)

            timestamp = int(parse(date).epoch)

            transactions.append({
                'timestamp': timestamp,
                'description': description,
                'amount': amount,
                'type': transaction_type,
                # 'code': code,
                # 'balance': balance,
            })

        return transactions

    # Step-by-step Functions

    def load(self):
        if self.is_ready:
            return

        self.driver = webdriver.PhantomJS()
        self.driver.set_window_size(1024, 768)
        self.driver.get('https://www1.bmo.com/onlinebanking/cgi-bin/netbnx/NBmain?product=5')

        self.sign_in()

        self.save_account_data()
        return self.account_data

    def sign_in(self):
        logger.debug('sign_in')
        self.wait_for_text_on_page('Sign in to Online Banking')
        self.screenshot('sign_in')

        card_field = self.css('#siBankCard')
        card_field.send_keys(os.environ['BMO_CARD_NUMBER'])

        password_field = self.css('#regSignInPassword')
        password_field.send_keys(os.environ['BMO_PASSWORD'])

        self.screenshot('sign_in_filled')

        try:
            continue_btn = self.css('[widgetid="btnBankCardContinue"] [role="button"]')
        except selenium.common.exceptions.NoSuchElementException:
            continue_btn = self.css('[widgetid="btnBankCardContinueNoCache1"] [role="button"]')

        continue_btn.click()

        self.wait_for_text_on_page('Challenge Question', 'My Accounts')

        if self.text_on_page('Challenge Question'):
            self.challenge_question()
        elif self.text_on_page('My Accounts'):
            self.accounts()

    def challenge_question(self):
        logger.debug('challenge_question')
        self.wait_for_text_on_page('Challenge Question')
        self.screenshot('challenge_question')

        security_question = self.css('#lblregSecurityQuestion').text.lower()

        if 'cousin' in security_question:
            security_answer = os.environ['BMO_SECURITY_COUSIN']
        elif 'sibling' in security_question:
            security_answer = os.environ['BMO_SECURITY_SIBLING']
        elif 'born' in security_question:
            security_answer = os.environ['BMO_SECURITY_CITY']
        else:
            logger.info(self.page_source())
            raise Exception('Don\'t recognize security question: "%s"' % security_question)

        logger.debug('Answering "%s" with "%s"' % (security_question, security_answer))

        security_answer_field = self.css('#signInSecurityQuestion')
        security_answer_field.send_keys(security_answer)

        self.screenshot('challenge_question_filled')

        continue_btn = self.css('[widgetid="btnContinue"] [role="button"]')
        continue_btn.click()

        self.accounts()

    def accounts(self):
        logger.debug('accounts')

        self.wait_for_text_on_page('My Accounts')

        self.screenshot('accounts')

    def save_account_data(self):
        logger.debug('save_account_data')

        account_selector = '#BankAccounts tr td.tableContainer table tr'
        account_els = self.driver.find_elements_by_css_selector(account_selector)

        for el in account_els:
            account_num = el.find_element_by_css_selector('.accountNumber').text
            raw_balance = el.find_element_by_css_selector('.summaryBalance.totals').text

            balance = self.string_to_money(raw_balance)

            account = {
                'account_number': account_num,
                'balance': balance,
            }
            self.account_data.append(account)

        self.is_ready = True

    # Helper Functions

    def wait_for_text_on_page(self, *texts):
        ecs = []

        logger_text = ' OR '.join(['"%s"' % t for t in texts])
        logger.debug('waiting until body contains %s' % logger_text)

        for text in texts:
            ec = EC.text_to_be_present_in_element((By.CSS_SELECTOR, 'body'), text)
            ecs.append(ec)

        try:
            WebDriverWait(self.driver, 10).until(AnyEc(*ecs))
        except selenium.common.exceptions.TimeoutException:
            if (self.text_on_page('Service is temporarily unavailable') or
                    self.text_on_page('maintenance')):
                raise Scraper.MaintenanceException()

            logger.info(self.page_text())
            raise selenium.common.exceptions.TimeoutException(
                'Couldn\'t find %s on the page' % logger_text
            )

    def text_on_page(self, text):
        return text in self.page_text()

    def page_text(self):
        return self.driver.find_element_by_css_selector('body').text

    def page_source(self):
        return self.driver.page_source

    def css(self, selector):
        return self.driver.find_element_by_css_selector(selector)

    def screenshot(self, name):
        if logger.level == logging.DEBUG:
            directory = '%s/screenshots' % os.path.dirname(os.path.realpath(__file__))

            if not os.path.exists(directory):
                os.makedirs(directory)

            self.driver.save_screenshot('%s/%s.png' % (directory, name))

    def string_to_money(self, s):
        """
        Handles strings of the format "$77,234.58 USD "
        """
        s = s.strip()
        s = s.replace('$', '')
        parts = s.split(' ')

        if len(parts) != 2:
            raise ValueError('String has too many parts (split on spaces)')

        amount, currency = parts
        return Money(amount, currency)

    def quit(self):
        self.driver.quit()


class AnyEc:
    """
    Use with WebDriverWait to combine expected_conditions in an OR.
    From http://stackoverflow.com/a/16464305/2208702
    """
    def __init__(self, *args):
        self.ecs = args

    def __call__(self, driver):
        for fn in self.ecs:
            try:
                if fn(driver):
                    return True
            except:
                pass


def main():
    scraper = BMOScraper()
    print scraper.load()
    print scraper.load_transactions(os.environ['BMO_USD_ACCOUNT_NUMBER'])
    print scraper.load_transactions(os.environ['BMO_CAD_ACCOUNT_NUMBER'])
    scraper.quit()

if __name__ == '__main__':
    main()



# from http://stackoverflow.com/a/952952/2208702
def flatten(l):
    return list([item for sublist in l for item in sublist])

def distinct(iterable, keyfunc=None):
    seen = set()
    for item in iterable:
        key = item if keyfunc is None else keyfunc(item)
        if key not in seen:
            seen.add(key)
            yield item



from collections import defaultdict
from datetime import datetime
import json
import uuid

from cdecimal import Decimal
from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, Numeric

from gryphon.lib.models.base import Base
from gryphon.lib.session import commit_mysql_session
from gryphon.lib.singleton import Singleton

metadata = Base.metadata


class Datum(Base):
    __tablename__ = 'datum'

    unique_id = Column(Unicode(64), nullable=False)
    datum_id = Column(Integer, primary_key=True)
    time_created = Column(DateTime, nullable=False)
    datum_type = Column(Unicode(256), nullable=False)
    numeric_value = Column(Numeric(precision=20, scale=10))
    string_value = Column(Unicode(256))
    meta_data = Column(UnicodeText(length=2**31))

    order_id = Column(Integer, ForeignKey('order.order_id'), nullable=True)

    def __init__(self, datum_type, numeric_value=None, string_value=None, meta_data={}, order=None):
        self.time_created = datetime.utcnow()
        self.datum_type = datum_type
        self.numeric_value = numeric_value
        self.string_value = string_value
        self.unique_id = u'dat_%s' % unicode(uuid.uuid4().hex)
        self.meta_data = json.dumps(meta_data)
        self.order = order

    def __unicode__(self):
        return unicode(repr(self))

    def __repr__(self):
        d = {
            'datum_type': self.datum_type,
            'time_created': unicode(self.time_created),
            'meta_data': json.loads(self.meta_data),
        }
        if self.numeric_value:
            d.update({'numeric_value': str(self.numeric_value)})
        if self.string_value:
            d.update({'string_value': self.string_value})

        return json.dumps(d, ensure_ascii=False)


class DatumRecorder(object):
    __metaclass__ = Singleton

    def create(self, db=None, logger=None):
        self.db = db
        self.external_logger = logger
        self.data_for_mean = defaultdict(list)

    def record(self, datum_type, numeric_value=None, string_value=None, meta_data={}, order=None):
        datum = Datum(
            datum_type,
            numeric_value=numeric_value,
            string_value=string_value,
            meta_data=meta_data,
            order=order,
        )

        if not hasattr(self, 'db') and not hasattr(self, 'external_logger'):
            raise Exception('DatumRecorder must be created before you can record')

        if self.db:
            self.db.add(datum)
            commit_mysql_session(self.db)
        elif self.external_logger:
            self.external_logger.info(datum)
        else:
            # we aren't recording events.
            pass

    def record_mean(self, datum_type, numeric_value, sample_size):
        """
        Store a datum with the mean of every <sample_size> data points.

        This has weird behaviour if two places call it with different sample_sizes.
        We should only have one DatumRecorder line per datum_type
        """
        if not hasattr(self, 'data_for_mean'):
            raise Exception('DatumRecorder must be created before you can record')

        data = self.data_for_mean[datum_type]
        data.append(numeric_value)

        if len(data) >= sample_size:
            mean = Decimal(sum(data)) / Decimal(len(data))
            self.record(datum_type, numeric_value=mean)
            # Clear the content of a referenced list
            del data[:]



"""
This is a minimal script that demonstrates authentication on itbit. This is useful for
debugging if you ever run into issues with authenticating on itbit.
"""

import base64
import hashlib
import hmac
import json
import time
import urllib
import os
import requests


USER_ID = os.environ['ITBIT_USER_ID']
API_KEY = os.environ['ITBIT_API_KEY']
API_SECRET = os.environ['ITBIT_API_SECRET'].encode('utf-8')
WALLET_ID = os.environ['ITBIT_WALLET_ID']


def main(script_arguments, execute):
    request_args = {}

    request_args['data'] = ''
    timestamp = int(round(time.time() * 1000))
    nonce = 1
    url = '/wallets/%s' % WALLET_ID
    req_method = 'GET'
    json_body = ''

    message = json.dumps([
        req_method.upper(),
        url,
        json_body,
        str(nonce),
        str(timestamp)
        ], separators=(',', ':'),
    )

    sha256_hash = hashlib.sha256()
    nonced_message = str(nonce) + message
    sha256_hash.update(nonced_message)
    hash_digest = sha256_hash.digest()

    msg_to_hmac = url.encode('utf8') + hash_digest
    hmac_digest = hmac.new(API_SECRET, msg_to_hmac, hashlib.sha512).digest()

    sig = base64.b64encode(hmac_digest)

    headers = request_args['headers'] = {}

    headers['Authorization'] = API_KEY + ':' + sig
    headers['X-Auth-Timestamp'] = timestamp
    headers['X-Auth-Nonce'] = nonce
    headers['Content-Type'] = 'application/json'

    # Make the request

    full_url = 'https://api.itbit.com/v1' + url

    print requests.get(full_url, data=request_args).text




# -*- coding: utf-8 -*-
import os

import pika
from pika.adapters import twisted_connection
from twisted.internet import defer, reactor, protocol, task
from twisted.python import log


class TwistedProducer(object):
    """
    This is an example producer that will handle unexpected interactions
    with RabbitMQ such as channel and connection closures.

    If RabbitMQ closes the connection, it will reopen it. You should
    look at the output, as there are limited reasons why the connection may
    be closed, which usually are tied to permission related issues or
    socket timeouts.

    It uses delivery confirmations and illustrates one way to keep track of
    messages that have been sent and if they've been confirmed by RabbitMQ.
    """

    def __init__(self, exchange, exchange_type, queue, binding_key):
        """
        Setup the example producer object, passing in the URL we will use to connect to
        RabbitMQ.
        """

        self._connection = None
        self._channel = None
        self._deliveries = []
        self._acked = 0
        self._nacked = 0
        self._message_number = 0
        self._stopping = False
        self._closing = False
        self.ready = False

        self.exchange = exchange
        self.exchange_type = exchange_type
        self.queue_name = queue
        self.binding_key = binding_key

    def report_error(self, error):
        log.err('Error in Twisted Producer:%s' % str(error))

    def connect(self):
        """
        This method connects to RabbitMQ, returning the connection handle.  When the
        connection is established, the on_connection_open method will be invoked by
        pika. If you want the reconnection to work, make sure you set
        stop_ioloop_on_close to False, which is not the default behavior of this
        adapter.

        :rtype: pika.SelectConnection
        """

        credentials = pika.PlainCredentials(
            os.environ['AMPQ_USER'],
            os.environ['AMPQ_PASS'],
        )

        parameters = pika.ConnectionParameters(
            os.environ['AMPQ_HOST'],
            int(os.environ['AMPQ_PORT']),
            '/',
            credentials,
        )

        parameters = pika.ConnectionParameters()

        cc = protocol.ClientCreator(
            reactor,
            twisted_connection.TwistedProtocolConnection,
            parameters,
        )

        d = cc.connectTCP('localhost', 5672)
        d.addCallback(lambda protocol: protocol.ready)
        d.addCallback(self.on_connection_open)
        d.addErrback(self.report_error)

    def close_connection(self):
        """This method closes the connection to RabbitMQ."""

        log.msg('Closing connection')

        self._closing = True
        self._connection.close()

    def on_connection_closed(self, connection, reply_code, reply_text):
        """
        This method is invoked by pika when the connection to RabbitMQ is closed
        unexpectedly. Since it is unexpected, we will reconnect to RabbitMQ if it
        disconnects.

        :param pika.connection.Connection connection: The closed connection obj
        :param int reply_code: The server provided reply_code if given
        :param str reply_text: The server provided reply_text if given

        """
        self._channel = None

        if self._closing:
            self.ready = False
        else:
            log.err(
                'Connection closed, reopening in 5 seconds: (%s) %s',
                reply_code,
                reply_text,
            )

            self._connection.add_timeout(5, self.reconnect)

    @defer.inlineCallbacks
    def on_connection_open(self, connection):
        """
        This method is called by pika once the connection to RabbitMQ has been
        established. It passes the handle to the connection object in case we need it.

        :type unused_connection: pika.SelectConnection
        """

        try:
            self.ready = False

            self._connection = connection
            self._connection.add_on_close_callback(self.on_connection_closed)

            self._channel = yield connection.channel()
            self._channel.add_on_close_callback(self.on_channel_closed)

            self._exchange = yield self._channel.exchange_declare(
                exchange=self.exchange,
                type=self.exchange_type,
            )

            self._queue = yield self._channel.queue_declare(
                queue=self.queue_name,
                auto_delete=False,
                exclusive=False,
                durable=True,
            )

            yield self._channel.queue_bind(
                exchange=self.exchange,
                queue=self.queue_name,
                routing_key=self.binding_key,
            )

            self._channel.confirm_delivery(self.on_delivery_confirmation)
            self.ready = True

        except Exception as e:
            self.report_error(e)

    def reconnect(self):
        """
        Will be invoked by the IOLoop timer if the connection is closed. See the
        on_connection_closed method.
        """

        # Create a new connection
        self.connect()

    def on_channel_closed(self, channel, reply_code, reply_text):
        """
        Invoked by pika when RabbitMQ unexpectedly closes the channel. Channels are
        usually closed if you attempt to do something that violates the protocol, such
        as re-declare an exchange or queue with different parameters. In this case,
        we'll close the connection to shutdown the object.

        :param pika.channel.Channel: The closed channel
        :param int reply_code: The numeric reason the channel was closed
        :param str reply_text: The text reason the channel was closed
        """

        log.err('Channel was closed: (%s) %s' % (reply_code, reply_text))

        if not self._closing:
            self._connection.close()

    def on_delivery_confirmation(self, method_frame):
        """
        Invoked by pika when RabbitMQ responds to a Basic.Publish RPC command, passing
        in either a Basic.Ack or Basic.Nack frame with the delivery tag of the message
        that was published. The delivery tag is an integer counter indicating the
        message number that was sent on the channel via Basic.Publish. Here we're just
        doing house keeping to keep track of stats and remove message numbers that we
        expect a delivery confirmation of from the list used to keep track of messages
        that are pending confirmation.

        :param pika.frame.Method method_frame: Basic.Ack or Basic.Nack frame
        """

        confirmation_type = method_frame.method.NAME.split('.')[1].lower()

        if confirmation_type == 'ack':
            self._acked += 1
        elif confirmation_type == 'nack':
            self._nacked += 1

        self._deliveries.remove(method_frame.method.delivery_tag)

    @defer.inlineCallbacks
    def wait_until_ready(self):
        def offline_check_if_ready(self):
            return self.ready

        is_ready = self.ready

        while not is_ready:
            is_ready = yield task.deferLater(reactor, 1, offline_check_if_ready, self)

    def publish_message(self, message):
        """
        If the class is not stopping, publish a message to RabbitMQ, appending a list of
        deliveries with the message number that was sent. This list will be used to
        check for delivery confirmations in the on_delivery_confirmations method.
        """

        if not self.ready:
            raise ValueError('Producer not ready befor message published.')

        if self._stopping:
            log.msg('Producer is Stopping')
            return

        if not self._channel:
            log.err('No Channel Avaialble')
            return

        properties = pika.BasicProperties(
            app_id=u'example-producer',
            content_type=u'application/json',
            delivery_mode=2,
        )

        self._channel.basic_publish(
            self.exchange,
            self.binding_key,
            message,
            properties,
        )

        self._message_number += 1
        self._deliveries.append(self._message_number)

    def close_channel(self):
        """
        Invoke this command to close the channel with RabbitMQ by sending the
        Channel.Close RPC command.
        """

        log.msg('Closing the channel')

        if self._channel:
            self._channel.close()

    def run(self):
        """
        Run the example code by connecting.
        """

        self.connect()

        # Wait until ready.
        return task.deferLater(reactor, 1, self.wait_until_ready)

    def stop(self):
        """
        Stop the example by closing the channel and connection. We set a flag here so
        that we stop scheduling new messages to be published. The IOLoop is started
        because this method is invoked by the Try/Catch below when KeyboardInterrupt is
        caught. Starting the IOLoop again will allow the producer to cleanly disconnect
        from RabbitMQ.
        """

        log.msg('Stopping')

        self._stopping = True
        self.close_channel()
        self.close_connection()

        log.msg('Stopped')




"""
Manual accounting script. Takes in an exchange and an exchange order id, queries the
exchange for details about the order, and adds the order and any associated trades to
the trading database just as if it were done by a running strategy.

TODOs:
- We should have a more graceful way of handling actors, probably that doesn't involve
  human users writing them in at the command line. Maybe we could work out a way for
  them to communicate the strategy they wish to account for and then this script can
  grab the actor from the actual file.
- Alternatively, a simple check would be 'has this actor ever been used in the database
  before? If not, then it might be an error and double check with the user.
- This script should be more thoroughly tested for non-BTC pairs.
"""

import termcolor as tc
import time

from delorean import epoch
import sqlalchemy

from gryphon.execution.strategies.base import Strategy
from gryphon.execution.lib.exchange_color import exchange_color
from gryphon.lib import session
from gryphon.lib.exchange.exceptions import *
from gryphon.lib.exchange.exchange_factory import *
from gryphon.lib.logger import get_logger
from gryphon.lib.models.order import Order
from gryphon.lib.money import Money

logger = get_logger(__name__)

EXPECTED_ACTORS = [
    'MANUAL',
    'MULTIEXCHANGE_LINEAR',
    'SIMPLE_ARB',
    'TRIVIAL_MM',
    'TRIVIAL_MM_EXT',
]

ACTOR_WARNING = """\
This strategy actor is not for any builtin strategy. Make sure that you have it right\
(include case) or your strategy's position and trades will not be accurate!\
"""


def manual_accounting(exchange_name, order_id, actor, execute=False):
    if actor not in EXPECTED_ACTORS:
        logger.warning(tc.colored(ACTOR_WARNING, color='yellow'))

    db = session.get_a_trading_db_mysql_session()

    try:
        exchange = make_exchange_from_key(exchange_name)
        exchange_data = make_exchange_data_from_key(exchange_name, db)

        vol_currency_key = '%s_total' % exchange.volume_currency.lower()

        try:
            exchange.cancel_order(order_id)
        except CancelOrderNotFoundError:
            pass

        details = exchange.get_order_details(order_id)

        unit_price = details['fiat_total'] / details[vol_currency_key].amount

        try:
            order = db.query(Order)\
                .filter(Order.exchange_order_id == order_id)\
                .filter(Order._exchange_name == exchange.name)\
                .one()

            order.time_created = epoch(details['time_created']).naive
            action = 'Updated'
        except sqlalchemy.orm.exc.NoResultFound:
            order = Order(
                actor,
                details['type'],
                details[vol_currency_key],
                unit_price,
                exchange,
                order_id,
            )

            order.time_created = epoch(details['time_created']).naive
            order.exchange_rate = Money('1', exchange.currency).to('USD').amount
            action = 'Added'

        position_change, position_change_no_fees = order.was_eaten(details)

        old_balance = exchange_data.balance[exchange.volume_currency]

        for currency_code, position in position_change.iteritems():
            exchange_data.position[currency_code] += position
            exchange_data.balance[currency_code] += position

        logger.info(
            'Order: %s for %.4f @ %.2f',
            order.order_type,
            order.volume,
            order.price,
        )

        for trade in order.trades:
            logger.info(
                'Trade: %s for %.4f @ %.2f',
                trade.trade_type,
                trade.volume,
                trade.price,
            )

        if execute:
            db.add(order)
            db.add(exchange_data)
            session.commit_mysql_session(db)

            logger.info(tc.colored(
                '%s order #%s and its %s trade(s)' % (
                action,
                order_id,
                len(details['trades'])),
                'green',
            ))

            logger.info(tc.colored(
                'Updated balance from %s to %s' % (old_balance,
                exchange_data.balance[exchange.volume_currency]),
                'green',
            ))

        else:
            logger.info(tc.colored(
                'pass --execute to save this order to the db',
                'red',
            ))

    finally:
        db.remove()




# -*- coding: utf-8 -*-
import os
from gryphon.lib.models.emeraldhavoc.base import EmeraldHavocBase
import json
import uuid
from decimal import *
from collections import defaultdict

from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_factory import make_exchange_from_key
from gryphon.lib import gryphon_json_serialize
from datetime import datetime, date, timedelta
from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, Numeric, UniqueConstraint
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declarative_base

metadata = EmeraldHavocBase.metadata


class Trade(EmeraldHavocBase):
    __tablename__ = 'trade'

    trade_id = Column(Integer, primary_key=True)
    unique_id = Column(Unicode(64), nullable=False)
    exchange = Column(Unicode(64), nullable=False)
    timestamp = Column(DateTime, nullable=False)
    time_created = Column(DateTime, nullable=False)
    exchange_trade_id = Column(Unicode(64), nullable=True)
    source = Column(Unicode(64), nullable=False)

    _price = Column('price', Numeric(precision=20, scale=10))
    _price_currency = Column('price_currency', Unicode(3))
    _volume = Column('volume', Numeric(precision=20, scale=10))
    _volume_currency = Column('volume_currency', Unicode(3))

    __table_args__ = (
        UniqueConstraint(exchange, exchange_trade_id, name='uniq_exchange_exchange_trade_id'),
    )

    def __init__(self, price, volume, exchange, timestamp, exchange_trade_id, source='EXCHANGE'):
        self.unique_id = unicode(uuid.uuid4().hex)
        self.time_created = datetime.utcnow()
        self.timestamp = timestamp
        self.price = price
        self.volume = volume
        self.exchange = exchange
        self.exchange_trade_id = exchange_trade_id
        self.source = source

    def __unicode__(self):
        return u'[TRADE - Exchange:%s] Price:%s, Volume:%s BTC, Exchange:%s' % (
            self.exchange, self.price, self.volume, self.exchange)

    def __repr__(self):
        return self.to_json()

    def to_json(self):
        return json.dumps({
            'trade_id':self.trade_id,
            'timestamp':unicode(self.timestamp),
            'unique_id':self.unique_id,
            'exchange':self.exchange,
            'price':self.price,
            'volume':self.volume
        }, ensure_ascii=False)

    @property
    def volume(self):
        return Money(self._volume, self._volume_currency)

    @volume.setter
    def volume(self, value):
        self._volume = value.amount
        self._volume_currency = unicode(value.currency)

    @property
    def price(self):
        return Money(self._price, self._price_currency)

    @price.setter
    def price(self, value):
        self._price = value.amount
        self._price_currency = unicode(value.currency)



"""
A non-trivial market making strategy.
"""

from cdecimal import Decimal

from gryphon.execution.strategies.base import Strategy
from gryphon.execution.strategies.builtin.fundamental_value import native as native_midpoint
from gryphon.lib import configuration as configuration_lib
from gryphon.lib.money import Money
from gryphon.lib.exchange import exceptions as exchange_exceptions
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.order_sliding import slide_order


class Multiexchange_linear(Strategy):
    def __init__(self, db, harness, strategy_configuration=None):
        super(Multiexchange_linear, self).__init__(db, harness)

        # Immutable properties.
        self.volume_currency = 'BTC'
        self.exchange_name = 'BITSTAMP_BTC_USD'
        self._actor = 'MULTI'  # TODO: make sure this works.

        # Configurable properties with defaults.
        self.target_exchange = None

        self.trusted_exchanges = []

        self.spread = Decimal('0.10')
        self.base_volume = Money('0.005', 'BTC')
        self.max_position = Money('0.5', 'BTC')
        self.fundamental_depth = Money('5', 'BTC')
        self.primary_exchange_name = None
        self.fundamental_exchange_names = None
        self.midpoint_weights = None
        self.use_variable_sizing = False
        self.variable_sizing_spread = Decimal('0.03')
        self.variable_sizing_multiplier = Decimal('2')
        self.slide_jump = Money('0.01', 'USD')
        self.slide_ignore_volume = Money('0.00000001', 'BTC')
        self.max_spread = Money('1000', 'USD')

        if strategy_configuration:
            self.configure(strategy_configuration)

    def configure(self, strategy_configuration):
        super(Multiexchange_linear, self).configure(strategy_configuration)
        self.init_configurable('spread', strategy_configuration)
        self.init_configurable('base_volume', strategy_configuration)
        self.init_configurable('max_position', strategy_configuration)
        self.init_configurable('fundamental_depth', strategy_configuration)
        self.init_configurable('primary_exchange_name', strategy_configuration)
        self.init_configurable('fundamental_exchange_names', strategy_configuration)
        self.init_configurable('midpoint_weights', strategy_configuration)
        self.init_configurable('use_variable_sizing', strategy_configuration)
        self.init_configurable('variable_sizing_spread', strategy_configuration)
        self.init_configurable('variable_sizing_multiplier', strategy_configuration)
        self.init_configurable('slide_jump', strategy_configuration)
        self.init_configurable('slide_ignore_volume', strategy_configuration)

        self.init_primary_exchange()
        self.init_fundamental_exchanges()

        self.validate_configuration()

    def validate_configuration(self):
        assert sum(self.midpoint_weights.values()) == Decimal('1')

    def init_primary_exchange(self):
        self.primary_exchange = self.harness.exchange_from_key(
            self.primary_exchange_name,
        )

        # This causes us to always audit our primary exchange.
        self.target_exchanges = [self.primary_exchange.name]

    def init_fundamental_exchanges(self):
        self.fundamental_exchange_names = configuration_lib.parse_configurable_as_list(
            self.fundamental_exchange_names,
        )

        self.fundamental_exchanges = [
            self.harness.exchange_from_key(exchange_name)
            for exchange_name in self.fundamental_exchange_names
        ]

    def get_all_orderbooks(self):
        orderbooks = {}

        for exchange in self.fundamental_exchanges:
            ob = exchange.get_orderbook()
            orderbooks[exchange.name] = ob

        return orderbooks

    def calculate_global_midpoint(self, orderbooks):
        """
        Unweighted average of the midpoints.

        TODO features:
            - count failures

        TODO Iterations:
            - midpoint of the global orderbook
            - incorporate balance into this calculation
        """
        total = Decimal('0')
        final_midpoint = 0

        for exchange_name, orderbook in orderbooks.items():
            ob_midpoint = native_midpoint.calculate(orderbook, self.fundamental_depth)
            final_midpoint += self.midpoint_weights[exchange_name.lower()] * ob_midpoint

        midpoint = total / len(orderbooks)

        return final_midpoint

    def tick(self, current_orders):
        """
        V0.5:
            For a configurable exchange:
            For a list of exchange orderbooks to consider
            Get the orderbooks.
            Calculate the weighted global midpoint.
            Calculate the prices at a fixed spread from the midpoint.
            Calculate the volumes, given the prices, position, and max position.
            Increase the volumes if we can place our orders at a very wide spread.
            Place the orders.
        """
        self.primary_exchange.cancel_all_open_orders()

        bid_extra_data = {}
        ask_extra_data = {}

        orderbooks = self.get_all_orderbooks()
        midpoint = self.calculate_global_midpoint(orderbooks)

        bid_price = midpoint - (midpoint * self.spread)
        ask_price = midpoint + (midpoint * self.spread)

        bid_extra_data['fundamental_value'] = midpoint
        ask_extra_data['fundamental_value'] = midpoint

        bid_price = slide_order(
            Consts.BID,
            bid_price,
            orderbooks[self.primary_exchange.name],
            self.slide_ignore_volume,
            self.slide_jump,
            self.max_spread / 2,
        )

        ask_price = slide_order(
            Consts.ASK,
            ask_price,
            orderbooks[self.primary_exchange.name],
            self.slide_ignore_volume,
            self.slide_jump,
            self.max_spread / 2,
        )

        bid_volume, bid_extra_data, ask_volume, ask_extra_data = self.calculate_volumes(
            midpoint,
            bid_price,
            ask_price,
            bid_extra_data,
            ask_extra_data,
        )

        if bid_volume > 0:
            try:
                self.primary_exchange.limit_order(
                    Consts.BID,
                    bid_volume,
                    bid_price,
                    extra_data=bid_extra_data,
                )
            except exchange_exceptions.InsufficientFundsError:
                self.harness.log('Insuff funds error, skipping', color='red')

        if ask_volume > 0:
            try:
                self.primary_exchange.limit_order(
                    Consts.ASK,
                    ask_volume,
                    ask_price,
                    extra_data=ask_extra_data,
                )
            except exchange_exceptions.InsufficientFundsError:
                self.harness.log('Insuff funds error, skipping', color='red')

    def calculate_base_volumes(self):
        """
        We place orders of size [base_volume] unless we are within [base_volume] of our
        max position on that size. In that case, we only place the difference between
        our [current position] and [max_position].
        """
        bid_volume = self.base_volume
        ask_volume = self.base_volume

        if (self.position + self.base_volume) > self.max_position:
            bid_volume = self.max_position - self.position

        if (self.position - self.base_volume) < -self.max_position:
            ask_volume = self.max_position + self.position

        return bid_volume, ask_volume

    def calculate_volumes(self, midpoint, bid_price, ask_price, bid_extra_data, ask_extra_data):
        bid_volume, ask_volume = self.calculate_base_volumes()

        if self.use_variable_sizing is True:
            bid_volume, bid_extra_data = self.apply_variable_sizing(
                Consts.BID,
                bid_volume,
                bid_price,
                midpoint,
                bid_extra_data,
            )

            ask_volume, ask_extra_data = self.apply_variable_sizing(
                Consts.ASK,
                ask_volume,
                ask_price,
                midpoint,
                ask_extra_data,
            )

        return bid_volume, bid_extra_data, ask_volume, ask_extra_data

    def apply_variable_sizing(self, side, order_volume, price, midpoint, extra_data):
        initial_volume = order_volume
        real_spread = None

        if side == Consts.BID:
            real_spread = midpoint / price
        elif side == Consts.ASK:
            real_spread = price / midpoint

        if (real_spread - 1) > self.variable_sizing_spread:
            order_volume = order_volume * self.variable_sizing_multiplier

            extra_data['variable_sizing_volume'] = order_volume - initial_volume

        return order_volume, extra_data

    def mid_tick_logging(self, midpoint, bid_price, ask_price):
        self.harness.log('Prospective bid: '.ljust(20) + str(bid_price), color='blue')
        self.harness.log('Midpoint: '.ljust(20) + str(midpoint), color='blue')
        self.harness.log('Prospective ask: '.ljust(20) + str(ask_price), color='blue')



import datetime

from delorean import Delorean

from gryphon.lib import assets
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.models.exchange import Balance


def get_liabilities_series(db, start_time, end_time):
    liabilities_series = []

    current_time = start_time

    while current_time <= end_time:
        debts = assets.get_active_liabilities(db, current_time)
        debt_balance = sum([d.amount for d in debts], Balance())

        liabilities_series.append((current_time, debt_balance))

        current_time += datetime.timedelta(days=1)

    return liabilities_series


def get_assets_series(db, start_time, end_time):
    account_keys = []

    account_keys = exchange_factory.initialized_ledgers(db)

    # The burn account is essentially a negative balance. Don't add it to our assets.
    if 'BURN' in account_keys:
        account_keys.remove('BURN')

    starting_balance = assets.ledger_balance(
        db,
        end_time=start_time,
        include_pending=True,
        exchange_names=account_keys,
    )

    assets_series = []

    current_time = start_time
    current_balance = starting_balance

    assets_series.append((current_time, current_balance))

    last_time = current_time
    current_time += datetime.timedelta(days=1)

    while current_time <= end_time:
        balance_diff = assets.ledger_balance(
            db,
            start_time=last_time,
            end_time=current_time,
            include_pending=True,
            exchange_names=account_keys,
        )
        current_balance += balance_diff

        assets_series.append((current_time, current_balance))

        last_time = current_time
        current_time += datetime.timedelta(days=1)

    return assets_series


def calculate_net_assets_series(assets_series, liabilities_series):
    assert(len(assets_series) == len(liabilities_series))

    net_assets_series = []
    for asset_datapoint, liability_datapoint in zip(assets_series, liabilities_series):
        assets_time, assets_balance = asset_datapoint
        liabilities_time, liabilities_balance = liability_datapoint

        net_assets_balance = assets_balance - liabilities_balance
        assert(assets_time == liabilities_time)
        net_assets_series.append((assets_time, net_assets_balance))

    return net_assets_series


def convert_balance_series_to_usd(balance_series):
    usd_series = []
    for datapoint in balance_series:
        time, balance = datapoint
        # Warning: historical exchange rate doesn't handle today
        usd_balance = balance.total_usd_value(date=time)
        usd_series.append((time, usd_balance))

    return usd_series


def process_usd_series_for_flot(usd_series):
    flot_series = []
    for datapoint in usd_series:
        time, usd_balance = datapoint

        flot_timestamp = int(Delorean(time, 'UTC').epoch) * 1000
        flot_usd_balance = float(usd_balance.amount)
        flot_datapoint = (flot_timestamp, flot_usd_balance)

        flot_series.append(flot_datapoint)

    return flot_series



"""
This script modifies the database object and ledger for an exchange to bring both in
line with the exchange API's reported balance for this account.

It does this by calculating forcing exchange_account.balance to equal ledger_balance,
and then adding arbitrary deposit or withdrawal transactions to the ledger to make
the ledger match what the exchange API reports.

This is NOT intended to be used except on test databases or in extraordinary
circumstances. Adding arbitrary adjustments to the ledger can cause serious problems for
your business down the line.

This script does not wipe the trade or transaction history.

TODO: We need better language for the 'balance' field on the ExchangeData/Exchange
account database object. It's really just a cache.
"""

import pyximport; pyximport.install()

import os
import sys

from gryphon.lib import session
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money

logger = get_logger(__name__)

FIX_DESCRIPTION = "\nOur record of %s\'s %s balance is off from what the exchange reports by %s. We will add an arbitrary %s for %s to bring our ledger back into agreement with the exchange."


def align_ledger_and_balance_cache_for_exchange_account_in_currency(db, exchange, currency, execute):
    """
    The first step to fixing a balance mismatch between an exchange and the gryphon db
    is to make sure that the ledger balance (calculated from trades and transactions)
    equals the 'balance' field on the exchange account object.
    """
    exchange_account = exchange.exchange_account_db_object(db)

    ledger_balance = exchange_account.ledger_balance()

    if exchange_account.balance[currency] != ledger_balance[currency]:
        exchange_account.balance[currency] = ledger_balance[currency]

        logger.info(
            'Updated the balance cache for db to match the ledger in currency %s.' % (
            currency,
        ))

        db.add(exchange_account)
    else:
        logger.info('%s balance cache is in line with the ledger.' % currency)


def align_ledger_and_exchange_balance_in_currency(db, exchange, exchange_balance, currency, execute):
    exchange_account = exchange.exchange_account_db_object(db)
    db_balance = exchange_account.balance
    ledger_balance = exchange_account.ledger_balance()

    mismatch = ledger_balance[currency] - exchange_balance[currency]

    tx = None
    tx_amount = abs(mismatch)
    tx_type = None

    # Create the transactions.
    if mismatch < 0:  # Our db balance is too low, record a desposit.
        tx_type = Transaction.DEPOSIT
    elif mismatch > 0:  # Our db balance too high, record a withdrawal.
        tx_type = Transaction.WITHDRAWL
    else:
        logger.info('%s balance is good, nothing do to.' % currency)
        return

    if tx_type is not None:
        tx = Transaction(
            tx_type,
            Transaction.IN_TRANSIT,
            tx_amount,
            exchange_account,
            transaction_details={'notes': 'Arbitrary ledger adjustment.'},
            fee=Money('0', currency),
        )

        logger.info(
            FIX_DESCRIPTION % (
            exchange.name,
            currency,
            mismatch,
            'deposit' if tx_type == Transaction.DEPOSIT else 'withdrawal',
            tx_amount,
        ))

    db.add(tx)
    tx.complete()


def reset_balance_for_exchange_in_currency(db, exchange, exchange_balance, currency, execute=False):
    """
    This function brings the database balance (exchange_account.balance) in line with
    the ledger balance, and then adds transactions as necessary to bring the ledger
    balance in line with the exchange balance.
    """
    align_ledger_and_balance_cache_for_exchange_account_in_currency(
        db,
        exchange,
        currency,
        execute,
    )

    align_ledger_and_exchange_balance_in_currency(
        db,
        exchange,
        exchange_balance,
        currency,
        execute,
    )

    if execute is True:
        db.commit()
        logger.info('Committed')
    else:
        logger.info('Not committing because execute=False')


def fix_balance_issues_for_exchange(exchange_name, execute=False):
    db = session.get_a_trading_db_mysql_session()
    exchange = exchange_factory.make_exchange_from_key(exchange_name)

    if not exchange:
        raise Exception('No exchange %s' % exchange_name)

    exchange_balance = exchange.get_balance()

    reset_balance_for_exchange_in_currency(
        db,
        exchange,
        exchange_balance,
        exchange.currency,
        execute,
    )

    reset_balance_for_exchange_in_currency(
        db,
        exchange,
        exchange_balance,
        exchange.volume_currency,
        execute,
    )


def main(script_arguments, execute):
    exchange_name = script_arguments['exchange_name']

    fix_balance_issues_for_exchange(exchange_name, execute)




import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)


def CHMoF(d,c,h,l,o,v,tf):
    CHMF = []
    MFMs = []
    MFVs = []
    x = tf
    while x < len(d):
        PeriodVolume = 0
        volRange = v[x-tf:x]
        for eachVol in volRange:
            PeriodVolume += eachVol

        
        MFM = ((c[x]-l[x])-(h[x]-c[x]))/(h[x]-l[x])

        MFV = MFM*(PeriodVolume)

        MFMs.append(MFM)
        MFVs.append(MFV)
        x+=1

    y = tf
    while y < len(MFVs):
        PeriodVolume = 0
        volRange = v[y-tf:y]
        for eachVol in volRange:
            PeriodVolume += eachVol
            
        consider = MFVs[y-tf:y]
        #print consider
        tfsMFV = 0

        for eachMFV in consider:
            tfsMFV+=eachMFV

        tfsCMF = tfsMFV/PeriodVolume
        CHMF.append(tfsCMF)
        #print tfsCMF
        #time.sleep(555)
        y+=1
    #print len(CHMF)
    #print len(date[tf+tf:])
    return date[tf+tf:],CHMF
    

CHMoF(date,closep,highp,lowp,openp,volume,20)



import gryphon.lib; gryphon.lib.prepare()

import os

from delorean import Delorean
import ofxclient

from gryphon.lib.logger import get_logger
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money
from gryphon.lib.scrapers.base import Scraper

logger = get_logger(__name__)


class BoAScraper(Scraper):
    def __init__(self):
        self.statements = {}

    def load(self):
        institution = ofxclient.Institution(
            id='5959',
            org='HAN',
            url='https://eftx.bankofamerica.com/eftxweb/access.ofx',
            username=os.environ['BOA_USERNAME'],
            password=os.environ['BOA_PASSWORD'],
        )

        try:
            institution.authenticate()
        except ValueError as e:
            raise Scraper.MaintenanceException(e)

        account_data = []
        ofx_accounts = institution.accounts()
        for ofx_account in ofx_accounts:
            account_number = ofx_account.number
            statement = ofx_account.statement()

            # save these so that load_transactions can use them
            self.statements[account_number] = statement

            balance = Money(statement.balance, 'USD')

            account = {
                'account_number': account_number,
                'balance': balance,
            }
            account_data.append(account)

        return account_data

    def load_transactions(self, account_num):
        statement = self.statements[account_num]

        transactions = []
        for ofx_transaction in reversed(statement.transactions):

            amount = Money(ofx_transaction.amount, 'USD')
            if amount > 0:
                transaction_type = Transaction.DEPOSIT
            else:
                amount = abs(amount)
                transaction_type = Transaction.WITHDRAWL

            timestamp = int(Delorean(ofx_transaction.date, timezone='UTC').epoch)

            transactions.append({
                'timestamp': timestamp,
                'description': ofx_transaction.payee,
                'amount': amount,
                'type': transaction_type,
                # 'id': ofx_transaction.id,
            })

        return transactions

    def quit(self):
        # BMO Scraper needs to quit phantomjs, so we need it here to preserve a common interface
        pass


def main():
    scraper = BoAScraper()
    print scraper.load()
    print scraper.load_transactions(os.environ['BOA_MAIN_ACCOUNT_NUMBER'])
    scraper.quit()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from datetime import datetime
import json
import os
import uuid

from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, BigInteger, Numeric

from gryphon.lib.models.base import Base

metadata = Base.metadata


class Ticker(Base):
    __tablename__ = 'ticker'
   
    unique_id = Column(Unicode(64), nullable=False)
    ticker_id = Column(Integer, primary_key=True)
    exchange = Column(Unicode(256))
    data = Column(UnicodeText(length=2**31))
    time_retrieved = Column(DateTime, nullable=False)
 
    def __init__(self, exchange, data):
        self.time_retrieved = datetime.utcnow()
        self.exchange = exchange
        self.unique_id = u'tkr_%s' % unicode(uuid.uuid4().hex)
        self.data = json.dumps(data)
        
    def __unicode__(self):
        return unicode(repr(self))
        
    def __repr__(self):
        return json.dumps({
            'exchange':self.exchange,
            'data': self.data,
            'time_retrieved': unicode(self.time_retrieved),
        }, ensure_ascii=False)



"""
Just a few exercises for our configuration library.
"""
import pyximport; pyximport.install()
import os
import time

from cdecimal import Decimal

from gryphon.execution.lib import config_helper
from gryphon.lib.money import Money
from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange


class BaseConfiguration(object):
    def __init__(self, *args, **kwargs):
        super(BaseConfiguration, self).__init__(*args, **kwargs)
        self.unconfigured_exchange_object = self.exchange_class()
        self.exchange_name = self.unconfigured_exchange_object.name.lower()
        self.volume_currency = self.unconfigured_exchange_object.volume_currency
        self.price_currency = self.unconfigured_exchange_object.currency

    def test_simple(self):
        configuration = {
            'platform': {'emerald': True},
            'strategy': {},
            'exchanges': {
                self.exchange_name: {
                    'fiat_balance_tolerance': Money('1', self.price_currency),
                },
            }
        }
      
        exchange = self.exchange_class(configuration=configuration)

        assert exchange.fiat_balance_tolerance == Money('1', self.price_currency)
        assert exchange.use_cached_orderbook == True

    def test_emerald_config_override(self):
        configuration = {
            'platform': {'emerald': False},
            'strategy': {},
            'exchanges': {
                self.exchange_name: {
                    'emerald': True,
                },
            },
        }

        exchange = self.exchange_class(configuration=configuration)

        assert exchange.use_cached_orderbook == True

    def test_no_config_goes_to_defaults(self):
        configuration = {
            'platform': {},
            'strategy': {},
        }

        exchange = self.exchange_class(configuration=configuration)

        assert exchange.use_cached_orderbook == False
        assert exchange.market_order_fee == self.DEFAULT_MARKET_FEE
        assert exchange.fiat_balance_tolerance == self.DEFAULT_FIAT_TOLERANCE
        assert exchange.volume_balance_tolerance == self.DEFAULT_VOLUME_TOLERANCE

    def test_all(self):
        configuration = {
            'platform': {'emerald': True},
            'strategy': {},
            'exchanges': {
                self.exchange_name: {
                    'market_order_fee': Decimal('0.1134'),
                    'limit_order_fee': Decimal('0.5911'),
                    'fiat_balance_tolerance': Money('1.032', self.price_currency),
                    'volume_balance_tolerance': Money('2.221', self.volume_currency),
                    'min_order_size': Money('442', self.volume_currency),
                    'emerald': False,
                    'max_tick_speed': 33215,
                },
            },
        }

        exchange = self.exchange_class(configuration=configuration)

        assert exchange.market_order_fee == Decimal('0.1134')
        assert exchange.limit_order_fee == Decimal('0.5911')
        assert exchange.fiat_balance_tolerance == Money('1.032', self.price_currency)
        assert exchange.volume_balance_tolerance == Money('2.221', self.volume_currency)
        assert exchange.min_order_size == Money('442', self.volume_currency)
        assert exchange.use_cached_orderbook == False
        assert exchange.max_tick_speed == 33215



import dotenv
import logging
import os

import tornado.httpserver
import tornado.ioloop
from tornado.options import options
import tornado.web
import tornadotoad

# I believe setting the environment needs to happen first so any gryphon imports already
# have access.
from gryphon.lib import environment
environment.load_environment_variables()

import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

from gryphon.dashboards.routes import url_patterns
from gryphon.dashboards.settings import settings
from gryphon.lib import configuration
from gryphon.lib import session

logger = logging.getLogger(__name__)

options.port = 8080

if 'APP_PORT' in os.environ:
    options.port = int(os.environ['APP_PORT'])

GDS_ERROR = """\
Could not find a gds credential, orderbook and other dashboards won't be operational\
"""


class PentecostApp(tornado.web.Application):
    """
    The main init for the application. Starts the application, defines the db.
    """
    def __init__(self):
        tornado.web.Application.__init__(self, url_patterns, **settings)

        self.dashboard_db = session.get_a_dashboard_db_mysql_session()
        self.trading_db = session.get_a_trading_db_mysql_session()

        try:
            self.gds_db = session.get_a_gds_db_mysql_session()
        except KeyError as e:
            logger.info(GDS_ERROR)
            self.gds_db = None

        self.configuration = configuration.read_config_from_file('dashboards.conf')
        

def main():
    tornadotoad.register(
      api_key=os.environ.get('AIRBRAKE_API_KEY'),
      environment=os.environ.get('APP_ENV'),
    )

    app = PentecostApp() 
    http_server = tornado.httpserver.HTTPServer(app)
    http_server.listen(options.port)
    tornado.ioloop.IOLoop.instance().start()


if __name__ == '__main__':
    main()



import termcolor as tc
import time

from gryphon.lib import session 
from gryphon.lib.exchange.exchange_factory import make_exchange_data_from_key, make_exchange_from_key
from gryphon.lib.logger import get_logger
from gryphon.lib.money import Money
from gryphon.lib.models.transaction import Transaction

logger = get_logger(__name__)


def transaction_complete(exchange_name, currency):
    db = session.get_a_trading_db_mysql_session()
    try:
        exchange_data = make_exchange_data_from_key(exchange_name, db)
        tr = db.query(Transaction).filter_by(exchange=exchange_data).filter_by(_amount_currency=currency).filter_by(transaction_status=Transaction.IN_TRANSIT).order_by(Transaction.time_created).first()
        if tr:
            tr.complete()
            session.commit_mysql_session(db)
            if tr.transaction_type == Transaction.DEPOSIT:
                action = "deposit to"
            elif tr.transaction_type == Transaction.WITHDRAWL:
                action = "withdrawal from"
            logger.info(tc.colored("Recorded %s %s %s" % (tr.amount, action, exchange_name), "green"))
        else:
            logger.info(tc.colored("No Transaction of that currency found", "red"))
    finally:
        db.remove()
    

def withdraw_fiat(exchange_name, target_exchange_name, amount_str, deposit_amount_str, transaction_details):
    db = session.get_a_trading_db_mysql_session()
    try:
        exchange_data = make_exchange_data_from_key(exchange_name, db)
        target_exchange_data = make_exchange_data_from_key(target_exchange_name, db)
        amount = Money.loads(amount_str)
        if deposit_amount_str:
            deposit_amount = Money.loads(deposit_amount_str)
            exchange_data.record_fiat_withdrawal(target_exchange_data, amount, deposit_amount=deposit_amount, transaction_details=transaction_details)
        else:
            exchange_data.record_fiat_withdrawal(target_exchange_data, amount, transaction_details=transaction_details)
            
        session.commit_mysql_session(db)
        logger.info(tc.colored("Recorded %s withdrawal from %s" % (amount, exchange_name), "green"))
    finally:
        db.remove()

def withdraw(exchange_name, target_exchange_name, amount_str):
    db = session.get_a_trading_db_mysql_session()
    try:
        exchange_data = make_exchange_data_from_key(exchange_name, db)
        target_exchange_data = make_exchange_data_from_key(target_exchange_name, db)
        target_exchange = make_exchange_from_key(target_exchange_name)
        amount = Money.loads(amount_str)
        
        addr = target_exchange.current_deposit_address
        exchange_data.record_withdrawal(target_exchange_data, amount, addr)
        session.commit_mysql_session(db)
        logger.info(tc.colored("Recorded %s withdrawal from %s" % (amount, exchange_name), "green"))
    finally:
        db.remove()



"""
Simple functions to find the "midpoint" of an orderbook given various definitions.
"""

import gryphon.lib.metrics.quote as quote_lib
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money

DEFAULT_QUOTE_DEPTH = '10'


class OrderbookSizeException(Exception):
    pass


def get_midpoint_from_orderbook(orderbook, depth=None):
    """
    A simple function to find the midpoint at a given depth specified in the orderbook's
    volume currency.

    Returns a USD Money.
    """

    try:
        if depth is None:
            vol_currency = orderbook['asks'][0].volume.currency

            depth = Money(DEFAULT_QUOTE_DEPTH, vol_currency)

        bid_quote = quote_lib.price_quote_from_orderbook(
            orderbook,
            Consts.BID,
            depth,
        )['total_price'] / depth.amount

        ask_quote = quote_lib.price_quote_from_orderbook(
            orderbook,
            Consts.ASK,
            depth,
        )['total_price'] / depth.amount
    except:
        raise OrderbookSizeException

    return (bid_quote + ask_quote) / 2



import os
import setuptools


class CleanCommand(setuptools.Command):
    """
    Custom clean command to tidy up the project root, because even
        python setup.py clean --all
    doesn't remove build/dist and egg-info directories, which can and have caused
    install problems in the past.
    """
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


with open('README.md', 'r') as f:
    long_description = f.read()


setuptools.setup(
    name='gryphon-alpha',
    packages=setuptools.find_packages(),
    version='0.12.4',
    author='MacLeod & Robinson, Inc.',
    author_email='hello@tinkercorp.com',
    description='A framework for running algorithmic trading strategies on cryptocurrency markets.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='http://www.gryphonframework.org',
    classifiers=(
        'Programming Language :: Python :: 2.7',
        'Operating System :: OS Independent',
        'License :: Other/Proprietary License',
    ),
    entry_points={
        'console_scripts': [
            'gryphon-runtests=gryphon.tests.runtests:main',
            'gryphon-execute=gryphon.execution.app:main',
            'gryphon-cli=gryphon.execution.console:main',
            'gryphon-dashboards=gryphon.dashboards.app:main',
        ],
    },
    include_package_data=True,
    install_requires=[
        'alembic==0.6.0',
        'Babel==2.6.0',
        'backports.shutil-get-terminal-size==1.0.0',
        'cement==2.10.12',
        'certifi==2018.4.16',
        'chardet==3.0.4',
        'coinbase==1.0.4',
        'contextlib2==0.5.5',
        'Cython==0.20.1',
        'decorator==4.3.0',
        'Delorean>=1.0.0,<2',
        'enum34==1.1.6',
        'futures==3.2.0',
        'gryphon-cdecimal==2.3',
        'gryphon-money',  # Our fork of Python Money.
        'gryphon-pusherclient',  # Our duplicate of PythonPusherClient.
        'idna==2.7',
        'ipython==5.7.0',
        'ipython-genutils==0.2.0',
        'line-profiler==2.1.2',
        'Mako==1.0.7',
        'MarkupSafe==1.0',
        'mock==1.0.1',
        'more-itertools>=4.2.0,<5',
        'MySQL-python==1.2.5',
        'nose==1.3.7',
        'pathlib2==2.3.2',
        'pexpect==4.6.0',
        'pickleshare==0.7.4',
        'prompt-toolkit==1.0.15',
        'ptyprocess==0.6.0',
        'Pygments==2.2.0',
        'pylibmc>=1.5.2,<2',
        'python-dotenv==0.8.2',
        'pytz==2018.5',
        'raven==6.9.0',
        'rednose>=1.3.0,<2',
        'redis==2.10.6',
        'requests==2.19.1',
        'requests-futures==0.9.7',
        'requests-toolbelt==0.8.0',
        'retrying==1.3.3',
        'scandir==1.7',
        'simplegeneric==0.8.1',
        'six==1.11.0',
        'sure==1.2.9',
        'SQLAlchemy>=1.2.10,<1.3',
        'termcolor==1.1.0',
        'traitlets==4.3.2',
        'tzlocal==1.5.1',
        'urllib3==1.23',
        'wcwidth==0.1.7',
        'websocket-client==0.48.0',
    ],
    cmdclass={
        'clean': CleanCommand,
    },
)



from delorean import Delorean
import tornado.web

from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


class StartEnd(tornado.web.UIModule):
    def render(self, start_time, end_time):
        formatted_start_time = self.format_date_for_title(start_time)
        formatted_end_time = self.format_date_for_title(end_time)

        prev_url, next_url = self.get_prev_and_next_url(start_time, end_time)

        current_day_url = self.get_current_day_url()
        current_month_url = self.get_current_month_url()

        return self.render_string(
            'templates/ui_modules/start_end.html',
            formatted_start_time=formatted_start_time,
            formatted_end_time=formatted_end_time,
            prev_url=prev_url,
            next_url=next_url,
            current_day_url=current_day_url,
            current_month_url=current_month_url,
        )

    def get_prev_and_next_url(self, start_time, end_time):
        prev_start, next_end = self.get_prev_start_and_next_end(start_time, end_time)

        prev_end = start_time
        prev_url = self.make_start_end_url(prev_start, prev_end)

        next_start = end_time
        next_url = self.make_start_end_url(next_start, next_end)

        return prev_url, next_url

    def get_current_day_url(self):
        start = Delorean().truncate('day')
        start_time = start.datetime
        end_time = start.next_day(1).datetime

        return self.make_start_end_url(start_time, end_time)

    def get_current_month_url(self):
        start = Delorean().truncate('month')
        start_time = start.datetime
        end_time = start.next_month(1).datetime

        return self.make_start_end_url(start_time, end_time)

    def make_start_end_url(self, start, end):
        args = {
            'start': self.format_date_for_url(start),
            'end': self.format_date_for_url(end),
        }

        base_url = self.request.path
        url = tornado.httputil.url_concat(base_url, args)
        return url

    def format_date_for_title(self, date):
        if date.hour == 0 and date.minute == 0 and date.second == 0:
            return date.strftime('%B %d, %Y')
        else:
            return date.strftime('%B %d, %Y %H:%M:%S')

    def format_date_for_url(self, date):
        if date.hour == 0 and date.minute == 0 and date.second == 0:
            return date.strftime('%Y-%m-%d')
        else:
            return str(date)

    def get_prev_start_and_next_end(self, dt_start, dt_end):
        start = Delorean(dt_start, 'UTC')
        end = Delorean(dt_end, 'UTC')

        if start.next_year(1) == end:
            prev_start = start.last_year(1)
            next_end = end.next_year(1)
        elif start.next_month(1) == end:
            prev_start = start.last_month(1)
            next_end = end.next_month(1)
        elif start.next_day(1) == end:
            prev_start = start.last_day(1)
            next_end = end.next_day(1)
        else:
            diff = dt_end - dt_start
            prev_start = dt_start - diff
            next_end = dt_end + diff
            # Returning here since we already have datetimes in this case
            return prev_start, next_end

        return prev_start.datetime, next_end.datetime



from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller


class QuadrigaOrderbook(OrderbookPoller):
    def __init__(self):
        self.exchange_name = u'QUADRIGA_BTC_CAD'
        self.url = 'https://api.quadrigacx.com/v2/order_book'
        self.poll_time = 2

    # API response format:
    # {"bids":[["302.50","1.03272000"], ...

    def parse_order(self, order):
        return [order[0], order[1], '']



# -*- coding: utf-8 -*-
import time
import os
import datetime
import logging
import json
import math
import uuid
import copy
from gryphon.lib.money import Money
from gryphon.lib.exchange.base import Exchange
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.models.exchange import Balance

from cdecimal import *
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class SimExchange(Exchange):
    
    def __init__(self, data_source=None, debug=False, engine=None):
        self.name = u'SIM'
        self.friendly_name = u'Sim'
        self.currency = u'USD'
        self.fee = Decimal("0.0000")
        
        if not engine:
            raise("Please pass an Engine.")
        self.engine = engine
        self.order_ids_buffer = []
     
    def fill_order_book(self, num_ticks=500):
        for i in range(num_ticks):
            self.engine._tick(self)
       
    # override this because we take the orders out in the engine
    def remove_our_orders(self, order_book, open_orders):
        pass


    ###### Common Exchange Methods ######

    # modifies request_args
    def auth_request(self, req_method, url, request_args):
        return None

    def balance_req(self):
        account_balance = self.engine.balance()
        logger.info('account_balance %s ' % account_balance)
        usd_balance = account_balance['usd']
        btc_balance = account_balance['btc']
        logger.info(u'[Sim Exchange] Balance: %s BTC %s USD' % (btc_balance, usd_balance))
        balance = Balance()
        balance['BTC'] = btc_balance
        balance['USD'] = usd_balance
        return balance

    def balance_resp(self, req):
        return req
    
    def get_order_book_req(self, debug=False, verify=True):
        # get the iteration number
        orderbook = self.engine.order_book(self, without_user_orders=True, debug=debug)
        return orderbook
       
    def get_order_book_resp(self, req, volume_limit=None):
        return req

    def create_trade_req(self, mode, volume, price):
        order = None

        #print volume

        if mode == Consts.ASK:
            order = self.engine.ask(volume, price, self)
        elif mode == Consts.BID:
            order = self.engine.bid(volume, price, self)
        else:
            raise ValueError('mode must be Consts.ASK or Consts.BID')
  
        return {
            'success':True,
            'order_id':order.order_id
        }
        
        
    def create_trade_resp(self, req):
        return req
    
    
    def open_orders_req(self):
        open_orders = self.engine.open_orders()
        open_orders_dicts = []
        for o in open_orders:
            order_type = o.type
            open_orders_dicts.append({
                'mode':order_type,
                'id':o.order_id,
                'price':o.price,
                'volume_remaining':o.volume
            })
        return open_orders_dicts
        
    def open_orders_resp(self, req):
        return req

    def multi_order_status_req(self):
        return None
        
    def multi_order_status_resp(self, req, order_ids):
        status_dict = {}
        for order_id in order_ids:
            order = self.engine.order_status(order_id)
            status = ''
            if order.status == Order.OPEN:
                status = 'open'
            elif order.status == Order.PARTIALLY_FILLED:
                status = 'open'
            elif order.status == Order.CANCELLED:
                status = 'cancelled'
            elif order.status == Order.FILLED:
                status = 'filled'
            else:
                status = 'error'
            status_dict[order_id] = {'status': status}
        return status_dict
            
    def order_details_req(self, order_id):
        self.order_ids_buffer.append(order_id)
        return None
    
    def order_details_resp(self):
        order_id = self.order_ids_buffer.pop()
        order = self.engine.order_status(order_id)
        if not order:
            raise Exception('You asked for an order that does not exist.')
        
        trades_dicts = []
        trades = self.engine.get_trades(order_id)
        fiat_total = Money(0, 'USD')
        for trade in trades:
            fiat_trade = trade.price * trade.volume.amount 
            fiat_total += fiat_trade
            trades_dicts.append({
                'trade_id':None,
                'fee': Money(0, 'USD'),
                'btc': trade.volume,
                'fiat': fiat_trade
            })
        btc_total = sum([t.volume for t in trades], Money(0, 'BTC'))
        order_type = order.type
        order_details = {
            'btc_total': btc_total,
            'fiat_total': fiat_total,
            'trades': trades_dicts,
            'type': order_type
        }
        return order_details
    
    def multi_order_details_req(self):
        return None
    
    def multi_order_details_resp(self, req, order_ids):
        data = {}
        for oid in order_ids:
            self.order_details_req(oid)
            data[oid] = self.order_details_resp()
        return data

    def cancel_order_req(self, order_id):
        return self.engine.cancel_order(order_id)

    def cancel_order_resp(self, req):
        return {'success': True}


